<?php
// Exit if accessed directly
if ( ! defined( "ABSPATH" ) ) exit;
if ( ! class_exists( 'IMC_Init' ) ) :

class IMC_Init {
	
	public function __construct() 
	{
		//set_site_transient('update_plugins', null); // Just for testing to see if the available plugin update gets shown. IF THIS IS ON ACTUALL PLUGIN UPDATES MAY NOT WORK: WP error: Plugin update failed.
		// Load Classes -----------------------------------------------
		require_once(IMCE_EXT_DIR . '/imgmce/ImgMCE_Init.php');
		new ImgMCE();
		new IMC_Templates();
		new IMC_Activate();
		new IMC_AMP();

		// Load Elementor add-on
		require_once(IMCE_EXT_DIR . '/elementor-imgmce/elementor-imgmce.php');

		if ( function_exists('bbp_kses_allowed_tags') ) 
		{
			new IMC_BBPress();
		}
		
		// Actions -------------------------------------------------------
		add_action( 'wp_loaded', array(__CLASS__, 'register_scripts') );
		add_action( is_admin() ? 'admin_enqueue_scripts' : 'wp_enqueue_scripts' , array(__CLASS__, 'enqueue_scripts'));
		add_action( 'admin_menu', array(__CLASS__, 'register_admin_menu'));
		// @since v1.1.5 changed to init hook (instead of admin_init) (to enable the button on frontend editors too)
		add_action( 'init', array(__CLASS__, '_tinymce_button') );
		add_action('admin_init', array( __CLASS__, 'check_for_plugin_updates') );

		/**
		 * GUTENBERG
		 * Edit files are stored in the plugin: gb-imgmce-block
		 * 
		 * https://github.com/ahmadawais/create-guten-block
		 * npm start, npm run build
		*/
		if( function_exists('register_block_type'))
		{
			
			// Hook: Frontend assets.
			add_action( 'enqueue_block_assets', array(__CLASS__, 'gb_imgmce_block_assets' ));
			// Hook: Editor assets.
			add_action( 'enqueue_block_editor_assets', array(__CLASS__, 'gb_imgmce_block_editor_assets' ));
		}

		// Filters -------------------------------------------------------
		add_filter('_dn_user_role', array(__CLASS__, 'set_imgmce_user_role'), 10, 2);
	}
	



	/**
	 * Deactivate Adning
	 */
	public static function deactivate()
	{
		// Deregister Adning License
		$activation = get_option('imgmce_activation', '');
		if( !empty($activation))
		{
			$resp = IMC_Activate::deregister(array('license-key' => $activation['license-key']));
			return $resp['msg'];
		}
	}
	



	/**
	 *  GUTENBERG Integration
	 */
	public static function gb_imgmce_block_assets() {
		// Styles. Not needed as it's empty (we have no custom CSS for this block) 
		// and because it's empty it's giving a missing dependencies warning in Query Monitor.
		/*wp_enqueue_style(
			'imgmce-cgb-style-css', // Handle. imgmce_block-cgb-style-css
			IMCE_ASSETS_URL.'/js/dist/blocks.style.build.css',
			array( 'wp-blocks' ) // Dependency to include the CSS after it.
			// filemtime( plugin_dir_path( __DIR__ ) . 'dist/blocks.style.build.css' ) // Version: filemtime — Gets file modification time.
		);*/
	} // End function gb_imgmce_block_assets().
	
	
	/**
	 * Enqueue Gutenberg block assets for backend editor.
	 *
	 * `wp-blocks`: includes block type registration and related functions.
	 * `wp-element`: includes the WordPress Element abstraction for describing the structure of your blocks.
	 * `wp-i18n`: To internationalize the block's text.
	 *
	 * @since 1.0.0
	 */
	public static function gb_imgmce_block_editor_assets() 
	{
		$userdata = get_userdata(get_current_user_id());
		if( !empty($userdata))
		{
			$role = $userdata->roles[0];
			$settings = IMC_Main::load_settings();
			//print_r($settings['role'][$role]);

			if( array_key_exists('post_types', $settings['role'][$role]))
			{
				if( is_admin() )
				{
					global $current_screen;
					if( in_array($current_screen->post_type, $settings['role'][$role]['post_types']) ){
						// Scripts.
						
						wp_enqueue_script(
							'imgmce_block-cgb-block-js', // Handle. imgmce_block-cgb-block-js
							IMCE_ASSETS_URL.'/js/dist/blocks.build.js', // Block.build.js: We register the block here. Built with Webpack.
							array( 'wp-editor', 'wp-blocks', 'wp-i18n', 'wp-element' ), // Dependencies, defined above.
							// filemtime( plugin_dir_path( __DIR__ ) . 'dist/blocks.build.js' ), // Version: filemtime — Gets file modification time.
							true // Enqueue the script in the footer.
						);
					
						// Styles.
						wp_enqueue_style(
							'imgmce_block-cgb-block-editor-css', // Handle. imgmce_block-cgb-block-editor-css
							IMCE_ASSETS_URL.'/js/dist/blocks.editor.build.css', // Block editor CSS.
							array( 'wp-edit-blocks' ) // Dependency to include the CSS after it.
							// filemtime( plugin_dir_path( __DIR__ ) . 'dist/blocks.editor.build.css' ) // Version: filemtime — Gets file modification time.
						);
					}
				}
			}
		}
	}
	



	/*
	 * register_scripts
	 *
	 * @access public
	 * @return null
	*/
	public static function register_scripts()
	{
		global $imc_scripts;

		$settings = IMC_Main::load_settings();

		ImgMCE::Init(array(
			'debug' => $settings['debug'],
			'imgmce_dir' => IMCE_EXT_DIR.'/imgmce/',
			'imgmce_url' => IMCE_EXT_URL.'/imgmce/',
			'upload_dir' => IMC_Main::upload_dir('basedir'),
			'upload_url' => IMC_Main::upload_dir('baseurl'),
			'siteurl' => get_site_url(),
			'current_user_id' => get_current_user_id(),
			'ajaxurl' => admin_url( 'admin-ajax.php' )
		));

		wp_register_style( 'imc_admin_css', IMCE_ASSETS_URL.'/css/imc_admin.css', false, IMCE_VERSION, "all" );

		// imgMCE scripts
		ImgMCE::register_scripts();
		//echo '<pre>'.print_r($imc_scripts->registered, true).'</pre>';
		if(!empty($imc_scripts->registered))
		{
			foreach( $imc_scripts->registered as $ext => $handles )
			{
				foreach( $handles as $key => $script )
				{
					if( $ext == 'js' )
					{
						$extra = !empty($script->extra) ? $script->extra : array();
						$in_footer = array_key_exists('group', $extra) ? true : false;
						$data = array_key_exists('json', $extra) ? $extra['json'] : array();
						
						wp_register_script( $key, $script->src, $script->deps, $script->ver, $in_footer );

						if(!empty($data))
						{
							wp_localize_script( $key, $key, $data );
						}
					}
					else
					{
						wp_register_style( $key, $script->src, $script->deps, $script->ver, 'all' );
					}
				}
				
			}
		}
		
	}



	/*
	 * enqueue_scripts
	 *
	 * @access public
	 * @return null
	*/
	public static function enqueue_scripts()
	{
		global $imc_scripts;

		// Scripts
		wp_enqueue_script('jquery');
		wp_enqueue_script('jquery-ui-core');
		wp_enqueue_script('jquery-ui-resizable');
		wp_enqueue_script('jquery-ui-sortable');

		// imgMCE scripts
		//echo '<pre>'.print_r($imc_scripts->registered, true).'</pre>';
		if(!empty($imc_scripts->registered))
		{
			foreach( $imc_scripts->registered as $ext => $handles )
			{
				foreach( $handles as $key => $script )
				{
					if( $ext == 'js')
					{
						wp_enqueue_script( $key );
					}
					else
					{
						wp_enqueue_style( $key );
					}
				}
			}
		}
	}


	/**
	 * Return the user role for the current user
	 * Filter function for: add_filter('_dn_user_role', array(__CLASS__, 'set_imgmce_user_role'), 10, 2);
	 */
	public static function set_imgmce_user_role($role = '', $uid = 0)
	{
		if( $uid )
		{
			$userdata = get_userdata($uid);
			$role = $userdata->roles[0];
		}

		return $role;
	}




	/*
	 * Add Admin Menu
	 * 
	*/
	public static function register_admin_menu()
	{	
		if( 
			isset( $_GET['page'] ) && $_GET['page'] == 'imgmce' || 
			isset( $_GET['page'] ) && $_GET['page'] == 'imgmce-settings' ||
			isset( $_GET['page'] ) && $_GET['page'] == 'imgmce-packages' || 
			isset( $_GET['page'] ) && $_GET['page'] == 'imgmce-role-manager' || 
			isset( $_GET['page'] ) && $_GET['page'] == 'imgmce-updates' 
		)
		{
			wp_enqueue_style('imc_admin_css');
			// Remove all admin notices
			remove_all_actions( 'admin_notices' );
		}

		// Create menu
		add_menu_page(
			__('imgMCE', 'imc'), 
			__('imgMCE', 'imc'), 
			IMCE_ACCESS_ROLE,  
			'imgmce', 
			array( __CLASS__, 'dashboard_template'),
			IMCE_ASSETS_URL.'/images/logo_20.png',
			80 
		);
		
		add_submenu_page('imgmce', 'imgMCE', 'imgMCE', IMCE_ACCESS_ROLE, 'imgmce', array( __CLASS__, "dashboard_template"));
		add_submenu_page("imgmce", __('General Settings', 'imc'), __('General Settings', 'imc'), IMCE_ACCESS_ROLE, "imgmce-settings", array( __CLASS__, "settings_template"));
		add_submenu_page("imgmce", __('Role Manager', 'imc'), __('Role Manager', 'imc'), IMCE_ACCESS_ROLE, "imgmce-role-manager", array( __CLASS__, "role_manager_template"));
		add_submenu_page("imgmce", __('Packages', 'imc'), __('Packages', 'imc'), IMCE_ACCESS_ROLE, "imgmce-packages", array( __CLASS__, "packages_template"));
		add_submenu_page("imgmce", __('Product License', 'imc'), __('Product License', 'imc'), IMCE_ACCESS_ROLE, "imgmce-updates", array( __CLASS__, "updates_template"));
	}

	// MENU FUNCTIONS -------------------------------------------------------
	public static function dashboard_template()
	{
		include( IMCE_TPL_DIR .'/dashboard.php');
	}
	public static function settings_template()
	{
		include( IMCE_TPL_DIR .'/settings.php');
	}
	public static function role_manager_template()
	{
		include( IMCE_TPL_DIR .'/role_manager.php');
	}
	public static function packages_template()
	{
		include( IMCE_TPL_DIR .'/packages.php');
	}
	public static function updates_template()
	{
		include( IMCE_TPL_DIR .'/updates.php');
	}
	



	/**
	 * TinyMCE Button for Wordpress text editors
	 */
	public static function _tinymce_button()
	{
		add_filter( 'mce_buttons', array(__CLASS__, 'register_tinymce_button') );
		add_filter( 'mce_external_plugins', array(__CLASS__, 'add_tinymce_button') );
	}

	public static function register_tinymce_button( $buttons ) 
	{
		$userdata = get_userdata(get_current_user_id());
		if( !empty($userdata))
		{
			$role = $userdata->roles[0];
			$settings = IMC_Main::load_settings();
			//echo '<pre>'.print_r($settings, true).'</pre>';
			
			if( array_key_exists('post_types', $settings['role'][$role]))
			{
				if( is_admin() )
				{
					global $current_screen;
					if(!empty($current_screen->post_type))
					{
						if( in_array($current_screen->post_type, $settings['role'][$role]['post_types']) ){
							array_push( $buttons, "ImgMCE" );
						}
					}
					else
					{
						array_push( $buttons, "ImgMCE" );
					}
				}
				else
				{
					array_push( $buttons, "ImgMCE" );
				}
			}
		}
		
		return $buttons;
   	}
   
	public static function add_tinymce_button( $plugin_array ) 
	{
		//$plugin_array['ImgMCE'] = IMC_ASSETS_URL. '/js/'.ADN_main::file_loc(array('name' => 'imgmce_tmce_plugin')).'.js' ;
		$plugin_array['ImgMCE'] = IMC_ASSETS_URL. 'dist/_imc_admin.bundle.js';

		return $plugin_array;
	}



	/*
	 * Plugin auto update
	 *
	 * @access public
	 * @return null
	*/
	public static function check_for_plugin_updates()
	{
		//set_site_transient('update_plugins', null); // Just for testing to see if the available plugin update gets shown. IF THIS IS ON ACTUALL PLUGIN UPDATES MAY NOT WORK: WP error: Plugin update failed.
		$activation = get_option('imgmce_activation', array());
		$license_key = !empty($activation) ? $activation['license-key'] : '';

		require( IMCE_CLASSES_DIR.'/IMGMCE_PLU_Auto_Plugin_Updater.php');
		$api_url = 'http://tunasite.com/updates/?plu-plugin=ajax-handler';
		// current plugin version | remote url | Plugin Slug (plugin_directory/plugin_file.php) | users envato license key (default: '') | envato item ID (default: '')
		new IMGMCE_PLU_Auto_Plugin_Updater(IMCE_VERSION, $api_url, IMCE_BASENAME.'/'.IMCE_BASENAME.'.php', $license_key, IMCE_ENVATO_ID);
	}
}
endif;
?>