<?php
/**
 * Usage:
 *
 * require_once(DIR. '/imgmce/ImgMCE.php');
 * new ImgMCE();
 * ImgMCE::enqueue_scripts();
 *
*/
if ( ! class_exists( 'ImgMCE' ) ) :

class ImgMCE {

	public static $current_user_id = 0; // ImgMCE::$current_user_id
	public static $current_user_role = '';

	public static function Init($args = array()) {
		global $imc_scripts;

		$defaults = array(
			'version' => '',
			'debug' => 1,
			'imgmce_dir' => '',
			'imgmce_url' => '',
			'upload_dir' => '',
			'upload_url' => '',
			'siteurl' => '',
			'current_user_id' => 0,
			'ajaxurl' => ''
		);
		$args = array_merge( $defaults, $args );

		self::$current_user_id = apply_filters('_dn_user_id', $args['current_user_id']);
		self::$current_user_role = apply_filters('_dn_user_role', '', $args['current_user_id']);

		define( "IMC_VERSION", '1.3.2' );
		define( "IMC_MAGICQUOTES", 1 );
		define( "IMC_DEBUG", $args['debug'] );
		define( "IMC_SITEURL", $args['siteurl']);
		define( "IMC_URL", $args['imgmce_url'] );
		define( "IMC_DIR", $args['imgmce_dir'] );
		define( "IMC_ASSETS_URL", IMC_URL. 'assets/' );
		define( "IMC_ASSETS_DIR", IMC_DIR. 'assets/' );
		define( 'IMC_INC_URL', IMC_URL. 'include/' );
		define( 'IMC_INC_DIR', IMC_DIR. 'include/' );
		define( 'IMC_AJAXURL', $args['ajaxurl'] );
		define( 'IMC_UPLOAD_FOLDER', 'imgMCE/');
		define( 'IMC_UPLOAD_DIR', $args['upload_dir'].'/'.IMC_UPLOAD_FOLDER);
		define( 'IMC_UPLOAD_SRC', $args['upload_url'].'/'.IMC_UPLOAD_FOLDER);


		// Load Classes -----------------------------------------------
		spl_autoload_register( array( __CLASS__, 'autoload' ) );

		
		$imc_scripts = new ADN_Scriptloader();
		new ADN_Main();
		new ADN_Banner();
		new ADN_Templates();
		new ADN_Shortcodes();
		//new ADN_API();
		new ADN_Scripts();
		new ADN_Ajax();
		new ADN_Google_Fonts();
		new ADN_Filters();
		new ADN_AMP();
		

		// Load Extensions -----------------------------------------------
		require_once(IMC_INC_DIR. 'extensions/imc_columns/IMC_Columns.php');
		new IMC_Columns();
		require_once(IMC_INC_DIR. 'extensions/imc_uploader/IMC_Uploader.php');
		new IMC_Uploader();
		require_once(IMC_INC_DIR. 'extensions/imc_colorpicker/IMC_ColorPicker.php');
		new IMC_ColorPicker();
		require_once(IMC_INC_DIR. 'extensions/imc_selectbox/IMC_SelectBox.php');
		new IMC_SelectBox();


		// Register scripts
		self::register_scripts();
	}



	/**
	 * Autoload classes
	 *
	 * @param string $class
	 */
	public static function autoload( $class ) 
	{
		// Not a Home class
		if ( 0 !== strncmp( 'ADN_', $class, 3 ) ) 
		{
			return;
		}

		$dirs = array(
			IMC_INC_DIR. 'classes/'
		);

		foreach ( $dirs as $dir ) 
		{
			if ( file_exists( $file = "$dir/$class.php" ) ) 
			{
				require_once $file;
				return;
			}
		}
	}





	/*
	 * register_scripts
	 *
	 * @access public
	 * @return null
	*/
	public static function register_scripts($args = array())
	{
		global $imc_scripts;

		$var_array = array(
		    'bgprem' => array( // bgprem* betalende gebruiker (premium)
		      'ngp' => 1 // ngp* "nog geen premium" ngp:1 = nog geen premium, ngp:0 = wel al premium.
			),
			'debug' => IMC_DEBUG,
			'ajaxurl' => IMC_AJAXURL,
			'siteurl' => IMC_SITEURL,
			'assets_url' => IMC_ASSETS_URL,
			'assets_dir' => IMC_ASSETS_DIR,
			'inc_url' => IMC_INC_URL,
			'inc_dir' => IMC_INC_DIR,
			'inc_dir_base' => ADN_Main::base_url(IMC_INC_DIR),
			'upload_folder' => IMC_UPLOAD_FOLDER,
			'upload_path' => IMC_UPLOAD_DIR,
			'upload_src' => IMC_UPLOAD_SRC
		);

		
		$imc_scripts->register_script( '_dn_', IMC_ASSETS_URL. 'dist/_imc.bundle.js', array( 'jquery' ), IMC_VERSION, true );
		$imc_scripts->localize( '_dn_', 'js', '_dn_', $var_array ); // 1.script handle, 2.variable name, 3.variable data
		
		$imc_scripts->register_script( '_imc_admin_bundle_js', IMC_ASSETS_URL. 'dist/_imc_admin.bundle.js', array( 'jquery' ), IMC_VERSION, true );
		//$imc_scripts->register_script( 'imgmce_image_js', IMC_ASSETS_URL. 'js/imgmce_image.js', array( 'jquery' ), IMC_VERSION, true );
		// Styles
		$imc_scripts->register_script( '_imc_style_css', IMC_ASSETS_URL.'dist/_imc.bundle.js.css', false, IMC_VERSION, "all" );
		$imc_scripts->register_script( '_imc_admin_style_css', IMC_ASSETS_URL.'dist/_imc_admin.bundle.js.css', false, IMC_VERSION, "all" );
		//$imc_scripts->register_script( '_dn_UI_style', IMC_ASSETS_URL.'css/lib/jqueryUI/jquery-ui.min.css', false, IMC_VERSION, "all" );
		
		
		IMC_ColorPicker::register_scripts(array('inc_url' => IMC_INC_URL, 'version' => IMC_VERSION));
		IMC_SelectBox::register_scripts(array('inc_url' => IMC_INC_URL, 'version' => IMC_VERSION));
		IMC_Columns::enqueue_scripts(array('inc_url' => IMC_INC_URL, 'version' => IMC_VERSION));
		IMC_Uploader::enqueue_scripts(array('inc_url' => IMC_INC_URL, 'version' => IMC_VERSION, 'prefix' => '_dn_spr'));
		
	}
}

endif;




/*
 * Default functions
 *
 * @access public
 * @return null
*/
function imgmce_head()
{
	global $imc_scripts;
	//print_r($imc_scripts);

	// first enqueue scripts
	if(!empty($imc_scripts->registered))
	{
		//print_r($imc_scripts->registered);
		foreach( $imc_scripts->registered as $ext => $handles )
		{
			foreach( $handles as $key => $script )
			{
				$imc_scripts->enqueue_script($key);
			}
			
		}
	}
	return $imc_scripts->print_scripts();
}


/**
 *  Translation functions
*/
if( !function_exists('__'))
{
    function __($text = '', $domain = '')
    {
        return ADN_Translation::translate(array('text' => $text, 'domain' => $domain));
    }
}
if( !function_exists('_e'))
{
    function _e($text = '', $domain = '')
    {
        echo ADN_Translation::translate(array('text' => $text, 'domain' => $domain));
    }
}
?>