/*
// USAGE EXAMPLE
<div id="froala-editor"></div>
<a id="submit">Submit</a>
<div id="result"></div>

jQuery(document).ready(function($){

    // Option to add custom variables to editor
    $.FroalaEditor.DEFAULTS = $.extend($.FroalaEditor.DEFAULTS, {
        imgMCE: {'save_to_folder': 0}
    });

    // Initialize the editor.
    $('#froala-editor').froalaEditor({
        toolbarButtons: ['bold', 'italic', '|', 'imgMCE'],
        pluginsEnabled: ['imgmcePlugin']
    });

    $('#submit').on('click', function(){
        $('#result').html( $('#froala-editor').froalaEditor('html.get') );
        $('#froala-editor').froalaEditor('html.set', '');
        ImgMCE_global.init_imgmce({'ajaxurl': _dn_.ajaxurl});
    });
});
*/


// https://www.froala.com/wysiwyg-editor/docs/concepts/custom/plugin
(function($, win) {

    // Make sure FroalaEditor is running on the page.
	if (typeof($.FroalaEditor) === "undefined") {
        if(parseInt(_dn_.debug)){ console.log('FroalaEditor is not available.'); }
		return;
    }
    
    
    $.FroalaEditor.PLUGINS.imgmcePlugin = function (editor) {

        var active_editor = editor.$box[0].id;

        // Detect dubble clicking on element
        $('#'+active_editor).find('.fr-element').on('dblclick','.imgmce_grid', function(){
            $('.imgmce_grid').removeClass('selected');
            $(this).addClass('selected');

            editor.opts = $.extend({imgMCE:{}}, editor.opts);
            var defaults = { 
                'active_editor': active_editor, 
                'in_popup':1,
                'save_to_folder': 1,
                'callback': "_imc_save_to_froala" 
            }
            editor.opts.imgMCE = $.extend(defaults, editor.opts.imgMCE);
            
            var code_obj = {};

            code_obj.id = typeof($(this).data('id')) !== 'undefined' ? $(this).data('id') : '',
            code_obj.user_id = typeof($(this).data('u')) !== 'undefined' ? $(this).data('u') : 0;

            if($(this).find('.imgmce_objcode_block').length){
                var thecode = JSON.parse(ImgMCE_global.urldecode($(this).find('.imgmce_objcode_block').html()));
                // Fix for older versions before v1.0.0
                if(typeof(thecode.banner_obj) !== 'undefined'){
                  thecode = thecode.banner_obj;
                }
                code_obj.banner_obj = thecode;
                if(parseInt(_dn_.debug)){ console.log(code_obj); }
            }

            code_obj.editing = 1;
            code_obj.editor = typeof(code_obj.editor) !== 'undefined' ? code_obj.editor : {};
            code_obj.editor.active_editor = active_editor;
            code_obj.editor.in_popup = editor.opts.imgMCE.in_popup;
            code_obj.editor.save_to_folder = editor.opts.imgMCE.save_to_folder;
            code_obj.editor.callback = editor.opts.imgMCE.callback;
            ImgMCE_global.load_editor(code_obj);
        });
      
        // Show the popup
        function showPopup () {

            editor.opts = $.extend({imgMCE:{}}, editor.opts);
            var defaults = { 
                'active_editor': active_editor, 
                'in_popup':1,
                'save_to_folder': 1,
                'callback': "_imc_save_to_froala" 
            }
            editor.opts.imgMCE = $.extend(defaults, editor.opts.imgMCE);

            ImgMCE_global.load_editor({ 'editor':editor.opts.imgMCE });
        }

        // Methods visible outside the plugin.
        return {
            showPopup: showPopup,
        }
    }

    // Define an icon and command for the button that opens the custom popup.
    $.FroalaEditor.DefineIconTemplate('imgmce_design', '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" viewBox="0 0 151 150.667" enable-background="new 0 0 151 150.667" xml:space="preserve"> <rect x="6.01" y="14.935" style="width:125.667px;height:101.667;" width="125.667" height="101.667"/> <path fill="none" stroke="#000000" stroke-width="3" stroke-miterlimit="10" d="M51.437,79.669"/> <path fill="none" stroke="#000000" stroke-width="3" stroke-miterlimit="10" d="M113.667,79.669"/> <path fill="none" stroke="#000000" stroke-width="3" stroke-miterlimit="10" d="M104,79.669"/> <path fill="#FFFFFF" d="M67.022,92.1l-6.352-4.808l14.021-14.119L47.062,44.454L9.5,76.845V18.335h118.833v70.282h-6.341 L95.76,64.946L67.022,92.1z"/> <circle cx="98.25" cy="40.586" r="9.417"/> <g id="XMLID_2_"> <g> <path fill="#F1E902" d="M39.271,86.771c3.358,0.867,7.099,1.496,10.216,2.513c-0.037-0.008-0.079-0.018-0.116-0.027 c-7.885,5.512-15.895,10.826-23.819,16.28c-0.105-0.003-0.099-0.071-0.089-0.143l0-0.001 C30.103,99.242,34.701,93.03,39.271,86.771z"/> </g> <g> </g> </g> <g id="XMLID_3_"> <g> <path fill="#F1E902" d="M57.173,83.715c-0.036,0.153-0.072,0.31-0.108,0.463c-2.583,1.667-5.068,3.486-7.605,5.222 c-6.518-1.525-13.037-3.05-19.554-4.574c0.025-0.114,0.053-0.232,0.08-0.348c4.268-6.962,8.502-13.979,12.669-21.1 c5.393,1.263,10.788,2.521,16.178,3.784c-4.416,4.585-8.858,9.126-13.184,13.857C49.491,81.918,53.331,82.816,57.173,83.715z"/> </g> <g> </g> </g> <g id="XMLID_5_"> <g> <path fill="#D4CC28" d="M33.656,85.58c1.862,0.436,3.725,0.872,5.589,1.307c-0.038,0.154-0.074,0.311-0.11,0.465 c-4.67,5.939-9.085,12.289-13.727,18.274c-1.829-0.59-4.413-0.711-5.765-1.595v-0.001C24.278,97.821,29.014,91.776,33.656,85.58z" /> </g> <g> </g> </g> <g id="XMLID_4_"> <g> <path fill="#D4CC28" d="M24.543,83.079c4.22-6.931,8.402-13.922,12.525-21.01c1.901,0.445,3.804,0.892,5.704,1.334 c-0.027,0.117-0.054,0.233-0.082,0.349c-4.22,6.933-8.402,13.924-12.525,21.014c-1.901-0.447-3.804-0.892-5.704-1.336 C24.488,83.312,24.516,83.195,24.543,83.079z"/> </g> <g> </g> </g> <path fill="#FFFFFF" d="M88.833,136c-4.135,0-7.498-3.363-7.498-7.497c0-4.135,3.363-7.498,7.498-7.498H139.5 c4.134,0,7.497,3.363,7.497,7.498c0,4.134-3.363,7.497-7.497,7.497H88.833z"/> <path d="M142.666,128.503c0,1.748-1.417,3.166-3.166,3.166H88.833c-1.749,0-3.167-1.418-3.167-3.166l0,0 c0-1.749,1.418-3.167,3.167-3.167H139.5C141.249,125.336,142.666,126.754,142.666,128.503L142.666,128.503z"/> <path fill="#FFFFFF" d="M88.833,117.498c-4.134,0-7.497-3.363-7.497-7.497c0-4.135,3.363-7.498,7.497-7.498H139.5 c4.134,0,7.497,3.363,7.497,7.498c0,4.134-3.363,7.497-7.497,7.497H88.833z"/> <path d="M142.666,110.001c0,1.748-1.416,3.166-3.166,3.166H88.833c-1.748,0-3.166-1.418-3.166-3.166l0,0 c0-1.749,1.418-3.167,3.166-3.167H139.5C141.25,106.834,142.666,108.252,142.666,110.001L142.666,110.001z"/> <path fill="#FFFFFF" d="M88.833,99.597c-4.135,0-7.498-3.363-7.498-7.497s3.363-7.498,7.498-7.498H139.5 c4.134,0,7.497,3.364,7.497,7.498s-3.363,7.497-7.497,7.497H88.833z"/> <path d="M142.666,92.1c0,1.748-1.417,3.166-3.166,3.166H88.833c-1.749,0-3.167-1.418-3.167-3.166l0,0 c0-1.749,1.418-3.167,3.167-3.167H139.5C141.249,88.933,142.666,90.351,142.666,92.1L142.666,92.1z"/> <circle cx="126.75" cy="128.086" r="6.083"/> <circle cx="101.417" cy="110" r="6.083"/> <circle cx="119.916" cy="92.099" r="6.083"/></svg>');
    $.FroalaEditor.DefineIcon('imgmceIcon', { NAME: 'imgmce', template: 'imgmce_design'})
    $.FroalaEditor.RegisterCommand('imgMCE', {
        title: 'imgMCE Editor',
        icon: 'imgmceIcon',
        undo: false,
        focus: false,
        plugin: 'imgmcePlugin',
        callback: function () {
            this.imgmcePlugin.showPopup();
        }
    });

}(jQuery, window));



function _imc_save_to_froala(_obj, arr, settings, frola){

    var $ = jQuery;

    // Insert content in tinymce editor
    var editing = $('body').find('#TMCimgModal').data('editing'),
        load_code = settings.editor.return_all_code;
        active_editor = settings.editor.active_editor,
        msg = JSON.stringify(arr),
        debug = settings.editor.debug;

    if( editing ){		
        console.log('replace'); console.log(active_editor);
        $('#'+active_editor).find('.fr-element').find("[data-id='" + arr.banner_obj.id + "']").replaceWith('<p></p>'+ImgMCE_global.editor_grid_tpl({'json':msg, 'code':load_code, 'classes':'fr-fic fr-dib fr-deletable fr-draggable', 'style': 'display:block;'})+'<p></p>');  
    }else{
        console.log('insert into tinymce editor ID:'+active_editor);
        console.log( ImgMCE_global.editor_grid_tpl({'json':msg, 'code':load_code}) );
      
        $('#'+active_editor).froalaEditor('html.insert', '<p></p>'+ImgMCE_global.editor_grid_tpl({'json':msg, 'code':load_code, 'classes':'fr-fic fr-dib fr-deletable fr-draggable', 'style': 'display:block;'})+'<p></p>');
    }
}


module.exports = { 
	_imc_save_to_froala
};