<?php
// Exit if accessed directly
if( !defined("IMC_VERSION")) exit;
if ( ! class_exists( 'ADN_Main' ) ) :

class ADN_Main {

	/*
	 * parse_args - Array Merge
	 * ADN_Main::parse_args( $args, $default );
	 *
	 * @access public
	 * @return array
	*/
	public static function parse_args($args, $default)
	{
		if(!is_array($args)) return $default;
		if(!is_array($default)) return $args;

		$is_multi = self::array_is_multi($default);
		$is_multi = !$is_multi ? self::array_is_multi($args) : $is_multi;

		return $is_multi ? self::array_merge_recursive_distinct($default, $args) : array_merge( $default, $args );
		//return array_merge( $default, $args );
	}

	// Check if array is multidimentional
	public static function array_is_multi($a) 
	{
		if(empty($a)) return false;
		foreach ($a as $v) 
		{
			if (is_array($v)) return true;
		}
		return false;
	}
	


	/**
     * Default IMAGE Settings, default banner_obj
     */
    public static function default_banner_obj()
    {
		$banner_obj = array(
			'v' => IMC_VERSION,
			'created' => time(),
			'align' => 'center',
			'size' => array( 
				'full' => '',
				'w' => '',
				'h' => ''
			), 
			'css' => array(
				'background-color' => 'transparent',
			)
		);

		$slides = array(
			'name' => __('Slide 1','imc'),
			'params' => array(
				'css' => array(
					'background-color' => 'transparent',
				)
			)
		);
		$slide_id = 0;
		$slides = self::parse_args($slides, array('slide_id' => $slide_id, 'slide_order' => 0));

		return self::parse_args($banner_obj, array('slides' => array($slides)));
	}



	/**
     * Default EDITOR Settings
     */
    public static function default_settings()
    {
        $default_settings = array(
            'image' => array(
				'image_uploads' => 1,
				'upload_size' => 15,
                'upload_file_types' => array('jpg','png','gif'),
                'stock_images' => 1,
                'stock_providers' => array(
                    'unsplash' => array(
                        'name' => 'Unsplash',
                        'url' => 'https://api.unsplash.com/',
                        'application_id' => '',
                    )
				),
				'giphy' => 1
			),
			'upload' => array(),
			'role' => array(),
			'menu' => array(
				'buttons' => array(
					'new_image' => 1,
					'my_images' => 1
				),
				'logo_url' => 'http://imgmce.com',
				'logo_title' => 'imgMCE - Easy to use, Professional, Animated, HTML5, Drag&Drop Element Builder'
			),
			'save_to_folder' => 1,
			'save_templates' => 0,
			'in_popup' => 0,
			'tinymce' => 0,
			'platform' => 'php',
            'active_editor' => '',
			'callback' => 0,
			'debug' => IMC_DEBUG,
			'messages' => array(
				'size_screen' => '',
				'editor_screen' => ''
			)
        );

        return $default_settings;
    }
    

    /**
	 * Load Settings
	*/
	public static function load_settings($args = array())
	{
		$settings = self::parse_args($args, self::default_settings());

		// Check if imgMCE Wordpress plugin exists and is activated.
		if( class_exists('IMC_Main') )
		{
			//$settings = self::parse_args($settings, IMC_Main::load_settings());
			$settings = self::parse_args(IMC_Main::load_settings(), $settings );
		}

        return $settings;
	}




	/**
	 * GET BASE URL from path
	 * path: /Applications/XAMPP/xamppfiles/htdocs/wordpress/wp-content/plugins/imgmce/include/extensions/imgmce
	 * base_url: /wordpress/wp-content/plugins/imgmce/include/extensions/imgmce
	 */
	public static function base_url($path)
	{
		$doc_root = preg_replace("!${_SERVER['SCRIPT_NAME']}$!", '', $_SERVER['SCRIPT_FILENAME']);
		return preg_replace("!^${doc_root}!", '', $path);
	}




	/**
	 * Return all local image plugin scripts
	 * Used for exporting images as HTML. If scripts are local we include them in the download package.
	 * 
	 * Called from ADN_Ajax::export_as_html5()
	 */
	public static function return_plugin_scripts($args = array())
	{
		$defaults = array(
			'banner_obj' => array()
		);
		$args = ADN_Main::parse_args($args, $defaults);
		$banner_obj = $args['banner_obj'];
		$scripts = array();

		if(!empty($banner_obj['slides']))
		{
			foreach ($banner_obj['slides'] as $key => $slide) 
			{
				if(!empty($slide['layers']))
				{
					foreach ($slide['layers'] as $key => $layer) 
					{
						// Check for plugins
						if( !empty($layer['plugins']))
						{
							foreach($layer['plugins'] as $pl => $plugin)
							{
								foreach ($plugin['external'] as $ext => $types)
								{
									if(!empty($types))
									{
										foreach($types as $type){
											if(!empty($type))
											{
												if( is_array($type) && !empty($type['path']))
												{
													$scripts[$pl][] = $type['path']['dir'].$type['path']['folder'].$type['path']['file'];
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}

		return $scripts;
	}



	/**
	 * SAVE (CREATE/UPDATE) file in folder
	*/
	public static function save_file_to_folder($args = array())
	{
		$defaults = array(
			'path' => '',
			'filename' => 'imgmce.json',
			'content' => ''
		);
		$args = ADN_Main::parse_args($args, $defaults);
		$path = rtrim($args['path'], '/');
		$path = $path.'/';

		if(! is_writable(dirname(dirname(dirname($path))))){
			return array('result' => 0, 'msg' => sprintf(__('Error: permission denied. %s','imc'), dirname(dirname(dirname($path)))));
		}
		if(! is_writable(dirname ($path))){
			return array('result' => 0, 'msg' => sprintf(__('Error: permission denied. %s','imc'), dirname($path)));
		}

		if(!empty($path) && !empty($args['content']))
		{
			if(!is_dir($path)) {
				mkdir($path, 0777, true);
			}

			// http://php.net/manual/en/function.fopen.php
			$fp = fopen($path.$args['filename'], 'w+');
			fwrite($fp, $args['content']);
			fclose($fp);

			return array('result' => '1 '.$path );
		}
	}



	/**
	 * Copy Folder
	 * https://stackoverflow.com/a/14156220/3481803
	 */
	public static function recurse_copy($src, $dst)
	{ 
		$dir = opendir($src);
		$result = ($dir === false ? false : true);
	
		if ($result !== false) 
		{
			$result = @mkdir($dst);
	
			if ($result === true) 
			{
				while(false !== ( $file = readdir($dir)) ) 
				{ 
					if (( $file != '.' ) && ( $file != '..' ) && $result) 
					{ 
						if ( is_dir($src . '/' . $file) ) 
						{ 
							$result = self::recurse_copy($src . '/' . $file,$dst . '/' . $file); 
						}
						else 
						{ 
							$result = copy($src . '/' . $file,$dst . '/' . $file); 
						}
					}
				}
				closedir($dir);
			}
			else
			{
				$result = sprintf(__('ERROR: Permission denied %s','imc'), $dst);
			}
		}
		else
		{
			$result = sprintf(__('ERROR: %s is not a folder','imc'), $src);
		}
	
		return $result;
	} 
	


	/**
	 * Load Library
	*/
	public static function load_library($args = array())
	{
        $defaults = array(
			'stickers' => array()
		);
		$library = ADN_Main::parse_args($args, $defaults);

		// Check if imgMCE Wordpress plugin exists and is activated.
		if( class_exists('IMC_Main') )
		{
			$library = IMC_Main::load_library();
		}
		
        return $library;
    }
	
	
	
	/*
	 * Link Masking
	 *
	 * @access public
	 * @return url
	*/
	public static function link_masking($id)
	{
		return get_bloginfo('url').'?_dnlink='.$id;
	}







	/*
	 * Image Sizes
	 *
	 * @access public
	 * @return array
	*/
	public static function image_sizes($args = array())
	{
		$defaults = array(
			'add_most_popular' => 1,
			'key' => ''
		);
		$args = ADN_Main::parse_args($args, $defaults);

		$sizes = array();

		// COMMON
		$sizes['common'] = array(
			'title' => 'Common Sizes',
			'sizes' => array(
				'590x300' => array(
					'title' => __('Envato product preview','imc'), 
					'size' => '590x300', 
					'pv' => '147x75', 
					'badge' => 'hot'
				),
				'1024x576' => array(
					'title' => __(' HDTV / DVD (16/9)','imc'), 
					'size' => '1024x576', 
					'pv' => '256x141', 
					'badge' => ''
				),
				'1024x768' => array(
					'title' => __(' LCD projectors (1 1/3)','imc'), 
					'size' => '1024x768', 
					'pv' => '256x192', 
					'badge' => ''
				)
			)
		);

		// BANNERS
		$sizes['banners'] = array(
			'title' => __('Banner Sizes','imc'),
			'sizes' => array(
				'728x90' => array(
					'title' => __('Leaderboard','imc'), 
					'size' => '728x90', 
					'pv' => '120x20', 
					'badge' => 'hot'
				),
				'300x250' => array(
					'title' => __('Medium Rectangle','imc'), 
					'size' => '300x250', 
					'pv' => '70x60', 
					'badge' => 'hot'
				),
				'336x280' => array(
					'title' => __('Large Rectangle','imc'), 
					'size' => '336x280', 
					'pv' => '84x70', 
					'badge' => '',
					//'premium' => 1
				),
				'250x250' => array(
					'title' => __('Square','imc'), 
					'size' => '250x250', 
					'pv' => '60x60', 
					'badge' => ''
				),
				'200x200' => array(
					'title' => __('Small Square','imc'), 
					'size' => '200x200', 
					'pv' => '50x50', 
					'badge' => 'new',
					'offset' => 70
				),
				'320x50' => array(
					'title' => __('Mobile','imc'), 
					'size' => '320x50', 
					'pv' => '75x15', 
					'badge' => ''
				),
				'120x600' => array(
					'title' => __('Skyscraper','imc'), 
					'size' => '120x600', 
					'pv' => '25x100', 
					'badge' => 'hot'
				),
				'160x600' => array(
					'title' => __('Wide Skyscraper','imc'), 
					'size' => '160x600', 
					'pv' => '32x100', 
					'badge' => ''
				),
				'240x600' => array(
					'title' => __('Extra Wide Skyscraper','imc'), 
					'size' => '240x600', 
					'pv' => '42x100', 
					'badge' => 'new'
				),
				'468x60' => array(
					'title' => __('Full Banner','imc'), 
					'size' => '468x60', 
					'pv' => '85x16', 
					'badge' => 'hot',
					'offset' => 70
				),
				'125x125' => array(
					'title' => __('Square Button','imc'), 
					'size' => '125x125', 
					'pv' => '30x30', 
					'badge' => '',
					'offset' => 85
				),
				'234x60' => array(
					'title' => __('Half Banner','imc'), 
					'size' => '234x60', 
					'pv' => '68x16', 
					'badge' => ''
				),
				'970x90' => array(
					'title' => __('Large Leaderboard','imc'), 
					'size' => '970x90', 
					'pv' => '120x20', 
					'badge' => ''
				),
				'120x240' => array(
					'title' => __('Vertical Banner','imc'), 
					'size' => '120x240', 
					'pv' => '30x56', 
					'badge' => ''
				),
				'728x300' => array(
					'title' => __('Pop Under','imc'), 
					'size' => '728x300', 
					'pv' => '120x70', 
					'badge' => ''
				)
			)
		);

		// SOCIAL MEDIA
		$sizes['social_media'] = array(
			'title' => __('Social Media Sizes','imc'),
			'sizes' => array(
				'851x315' => array(
					'title' => __('Facebook Cover','imc'), 
					'size' => '851x315',
					'pv' => '212x78', 
					'badge' => ''
				),
				'1500x500' => array(
					'title' => __('Twitter Cover','imc'), 
					'size' => '1500x500',
					'pv' => '375x125', 
					'badge' => ''
				),
				'1080x1080' => array(
					'title' => __('Instagram Post','imc'), 
					'size' => '1080x1080',
					'pv' => '180x180', 
					'badge' => ''
				),
				'1080x1920' => array(
					'title' => __('Instagram Story','imc'), 
					'size' => '1080x1920',
					'pv' => '180x320', 
					'badge' =>''
				)
			)
		);


		// Most Popular
		if($args['add_most_popular'])
		{
			$mp = array();
			$mp['most_popular'] = array('title' => __('Most Popular','imc'), 'sizes' => array());

			foreach($sizes as $size_cat)
			{
				foreach($size_cat['sizes'] as $key => $size)
				{
					if($size['badge'] == 'hot')
					{
						$mp['most_popular']['sizes'][$key] = $size;
					}
				}
			}

			// Add Most Popular to the beginning of the array
			$sizes = $mp+$sizes;
		}

		// Add Filter to adjust sizes  using the API
		$sizes = apply_filters('_dn_image_sizes', $sizes);

		return !empty($args['key']) ? $sizes[$args['key']] : $sizes;
	}




	/*
	 * Image Filters
	 *
	 * @access public
	 * @return array
	*/
	public static function _dn_filters($args = array())
	{
		$defaults = array(
			'add_most_popular' => 1,
			'key' => ''
		);
		$args = ADN_Main::parse_args($args, $defaults);

		$filters = array(
			'_normal' => array(
				'name' => 'Normal',
				'layer' => '',
				'css' => ''
			),
			'_1977' => array(
				'name' => '1977',
				'layer' => 'sepia(0%) brightness(110%) contrast(110%) saturate(130%) grayscale(0%) invert(0%) hue-rotate(0deg) blur(0px)',
				'css' => 'opacity:1;mix-blend-mode:screen;background:rgba(243, 106, 188, 0.3);'
			),
			'_alden' => array(
				'name' => 'Alden',
				'layer' => 'sepia(0%) brightness(120%) contrast(90%) saturate(85%) grayscale(0%) invert(0%) hue-rotate(20deg) blur(0px)',
				'css' => 'opacity: 1;mix-blend-mode:darken;background:linear-gradient(to right, rgba(66, 10, 14, 0.2) 1%, rgba(66, 10, 14, 0) 100%);'
			),
			'_hudson' => array(
				'name' => 'Hudson',
				'layer' => 'sepia(0%) brightness(120%) contrast(90%) saturate(110%) grayscale(0%) invert(0%) hue-rotate(0deg) blur(0px)',
				'css' => 'opacity:.5;mix-blend-mode:multiply;background:-webkit-radial-gradient(center center, circle closest-corner, rgb(255, 177, 166) 50%, rgb(52, 33, 52) 100%);background: transparent radial-gradient(circle closest-corner at center center , rgb(255, 177, 166) 50%, rgb(52, 33, 52) 100%) repeat scroll 0% 0%;'
			),
			'_inkwell' => array(
				'name' => 'Inkwell',
				'layer' => 'sepia(30%) brightness(110%) contrast(110%) saturate(100%) grayscale(100%) invert(0%) hue-rotate(0deg) blur(0px)',
				'css' => 'opacity: 1;mix-blend-mode:normal;background:rgba(0, 0, 0, 0);'
			),
			'_brannan' => array(
				'name' => 'Brannan',
				'layer' => 'sepia(50%) brightness(100%) contrast(140%) saturate(100%) grayscale(0%) invert(0%) hue-rotate(0deg) blur(0px)',
				'css' => 'opacity: 1;mix-blend-mode:lighten;background:rgba(161, 44, 199, 0.31);'
			),
			'_xpro2' => array(
				'name' => 'Xpro2',
				'layer' => 'sepia(30%) brightness(100%) contrast(100%) saturate(100%) grayscale(0%) invert(0%) hue-rotate(0deg) blur(0px)',
				'css' => 'opacity: 1;mix-blend-mode:color-burn;background:-webkit-radial-gradient(center center, circle closest-corner, rgb(224, 231, 230) 40%, rgba(43, 42, 161, 0.6) 100%);background:transparent radial-gradient(circle closest-corner at center center , rgb(224, 231, 230) 40%, rgba(43, 42, 161, 0.6) 100%) repeat scroll 0% 0%;'
			),
			'_amaro' => array(
				'name' => 'Amaro',
				'layer' => 'sepia(0%) brightness(110%) contrast(90%) saturate(150%) grayscale(0%) invert(0%) hue-rotate(-10deg) blur(0px)',
				'css' => 'mix-blend-mode: screen;opacity: 0.5;'
			),
			'_brooklyn' => array(
				'name' => 'Brooklyn',
				'layer' => 'sepia(0%) brightness(110%) contrast(90%) saturate(100%) grayscale(0%) invert(0%) hue-rotate(0deg) blur(0px)',
				'css' => 'mix-blend-mode:overlay;opacity:1;background: -webkit-radial-gradient(center center, circle closest-corner, rgba(168, 223, 193, 0.4) 1%, rgba(183, 196, 200, 0.2) 100%);background: transparent radial-gradient(circle closest-corner at center center , rgba(168, 223, 193, 0.4) 1%, rgba(183, 196, 200, 0.2) 100%) repeat scroll 0% 0%;'
			),
			'_clarendon' => array(
				'name' => 'Clarendon',
				'layer' => 'sepia(0%) brightness(100%) contrast(120%) saturate(125%) grayscale(0%) invert(0%) hue-rotate(0deg) blur(0px)',
				'css' => 'mix-blend-mode: overlay;opacity:1;background: rgba(127, 187, 227, 0.2);'
			),
			'_earlybird' => array(
				'name' => 'Earlybird',
				'layer' => 'sepia(20%) brightness(100%) contrast(90%) saturate(100%) grayscale(0%) invert(0%) hue-rotate(0deg) blur(0px)',
				'css' => 'mix-blend-mode: overlay;opacity:1;background:-webkit-radial-gradient(center center, circle closest-corner, rgb(208, 186, 142) 20%, rgba(29, 2, 16, 0.2) 100%);background: transparent radial-gradient(circle closest-corner at center center , rgb(208, 186, 142) 20%, rgba(29, 2, 16, 0.2) 100%) repeat scroll 0% 0%;'
			),
			'_gingham' => array(
				'name' => 'Gingham',
				'layer' => 'sepia(0%) brightness(105%) contrast(100%) saturate(100%) grayscale(0%) invert(0%) hue-rotate(350deg) blur(0px)',
				'css' => 'mix-blend-mode: darken;opacity:1;background:linear-gradient(to right, rgba(66, 10, 14, 0.2) 1%, rgba(0, 0, 0, 0) 100%);'
			),
			'_walden' => array(
				'name' => 'Walden',
				'layer' => 'sepia(30%) brightness(110%) contrast(100%) saturate(160%) grayscale(0%) invert(0%) hue-rotate(350deg) blur(0px)',
				'css' => 'mix-blend-mode: screen;opacity:.3;background:rgb(204, 68, 0);'
			),
			'_valencia' => array(
				'name' => 'Valencia',
				'layer' => 'sepia(8%) brightness(108%) contrast(108%) saturate(100%) grayscale(0%) invert(0%) hue-rotate(0deg) blur(0px)',
				'css' => 'mix-blend-mode: exclusion;opacity:.5;background:rgb(58, 3, 57);'
			),
			'_stinson' => array(
				'name' => 'Stinson',
				'layer' => 'sepia(0%) brightness(115%) contrast(75%) saturate(85%) grayscale(0%) invert(0%) hue-rotate(0deg) blur(0px)',
				'css' => 'mix-blend-mode: soft-light;opacity:.5;background:rgba(240, 149, 128, 0.2);'
			),
			'_reyes' => array(
				'name' => 'Reyes',
				'layer' => 'sepia(22%) brightness(110%) contrast(85%) saturate(75%) grayscale(0%) invert(0%) hue-rotate(0deg) blur(0px)',
				'css' => 'mix-blend-mode: soft-light;opacity:.5;background:rgb(173, 205, 239);'
			),
		);

		return $filters;
	}






	/*
	 * HTML Shapes
	 *
	 * @access public
	 * @return array
	*/
	public static function html_shapes($args = array())
	{
		$defaults = array(
			'linid' => rand(),
			'key' => '', 
		);
		$args = ADN_Main::parse_args($args, $defaults);

		$default_class = ' _dn_elem html_shape shape';

		$shapes = array(
			// Squares
			'square_solid' => array(
				'html' => '<div class="square'.$default_class.'" style="width:100%;height:100%;background-color:#000;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x100'
			),
			'square_border_1' => array(
				'html' => '<div class="square_border _bg_border'.$default_class.'" style="width:100%;height:100%;background-color:transparent;border:solid 1px #000000;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x100'
			),
			/*'square_border_3' => array(
				'html' => '<div class="square_border _bg_border'.$default_class.'" style="width:100%;height:100%;background-color:transparent;border:solid 3px #000000;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x100'
			),*/
			'square_border_6' => array(
				'html' => '<div class="square_border _bg_border'.$default_class.'" style="width:100%;height:100%;background-color:transparent;border:solid 6px #000000;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x100'
			),
			'square_border_10' => array(
				'html' => '<div class="square_border _bg_border'.$default_class.'" style="width:100%;height:100%;background-color:transparent;border:solid 10px #000000;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x100'
			),

			// Rounded Squares
			'square_solid_round' => array(
				'html' => '<div class="square'.$default_class.'" style="width:100%;height:100%;background-color:#000;border-radius:5px;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x100'
			),
			'square_border_1_round' => array(
				'html' => '<div class="square_border _bg_border'.$default_class.'" style="width:100%;height:100%;background-color:transparent;border:solid 1px #000000;border-radius:5px;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x100'
			),
			/*'square_border_3_round' => array(
				'html' => '<div class="square_border _bg_border'.$default_class.'" style="width:100%;height:100%;background-color:transparent;border:solid 3px #000000;border-radius:5px;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x100'
			),*/
			'square_border_6_round' => array(
				'html' => '<div class="square_border _bg_border'.$default_class.'" style="width:100%;height:100%;background-color:transparent;border:solid 6px #000000;border-radius:5px;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x100'
			),
			'square_border_10_round' => array(
				'html' => '<div class="square_border _bg_border'.$default_class.'" style="width:100%;height:100%;background-color:transparent;border:solid 10px #000000;border-radius:5px;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x100'
			),

			// Circles
			'circle_solid' => array(
				'html' => '<div class="circle'.$default_class.'" style="width:100%;height:100%;background-color:#000;border-radius: 50%;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x100'
			),
			'circle_border_1' => array(
				'html' => '<div class="circle _bg_border'.$default_class.'" style="width:100%;height:100%;background-color:transparent;border:solid 1px #000000;border-radius: 50%;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x100'
			),
			/*'circle_border_3' => array(
				'html' => '<div class="circle _bg_border'.$default_class.'" style="width:100%;height:100%;background-color:transparent;border:solid 3px #000000;border-radius: 50%;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x100'
			),*/
			'circle_border_6' => array(
				'html' => '<div class="circle _bg_border'.$default_class.'" style="width:100%;height:100%;background-color:transparent;border:solid 6px #000000;border-radius: 50%;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x100'
			),
			'circle_border_10' => array(
				'html' => '<div class="circle _bg_border'.$default_class.'" style="width:100%;height:100%;background-color:transparent;border:solid 10px #000000;border-radius: 50%;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x100'
			),

			// Lines
			'line_solid_1' => array(
				'html' => '<div class="line'.$default_class.'" style="width:100%;height:2px;background-color:#000;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x30'
			),
			'line_solid_4' => array(
				'html' => '<div class="line'.$default_class.'" style="width:100%;height:4px;background-color:#000;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x30'
			),
			'line_solid_6' => array(
				'html' => '<div class="line'.$default_class.'" style="width:100%;height:6px;background-color:#000;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x30'
			),
			'line_solid_dotted' => array(
				'html' => '<div class="line _bg_border'.$default_class.'" style="width:100%;height:2px;border:dashed 2px #000;"></div>',
				'aspect_ratio' => 0,
				'size' => '100x30'
			),


			// Pulse
			'round_pin_pulse' => array(
				'html' => '<div class="_dn_pin shape_ani"><span class="_dn_pin_pulse _dn_elem _dn_round"></span><span class="_dn_pin_icon _dn_elem _dn_round"><div class="_dn_pin_icon_content "></div></span></div>',
				'aspect_ratio' => 0,
				'size' => '60x60'
			),
			'square_pin_pulse' => array(
				'html' => '<div class="_dn_pin shape_ani"><span class="_dn_pin_pulse _dn_elem"></span><span class="_dn_pin_icon _dn_elem"><div class="_dn_pin_icon_content "></div></span></div>',
				'aspect_ratio' => 0,
				'size' => '60x60'
			),
		);

		return !empty($args['key']) ? $shapes[$args['key']] : $shapes;
	}



	/*
	 * Buttons
	 *
	 * @access public
	 * @return array
	*/
	public static function html_buttons($args = array())
	{
		$defaults = array(
			'linid' => rand(),
			'key' => '', 
		);
		$args = ADN_Main::parse_args($args, $defaults);

		$default_class = ' _dn_elem html_button shape';

		$shapes = array(
			'point-of-interest-red' => array(
				'html' => '<div class="poi-single-point poi-red"><a class="poi-img-replace" href="#0">More</a><div class="poi-more-info poi-top"></div></div>',
				'aspect_ratio' => 0,
				'size' => '30x30'
			),
			'point-of-interest-blue' => array(
				'html' => '<div class="poi-single-point poi-blue"><a class="poi-img-replace" href="#0">More</a><div class="poi-more-info poi-top"></div></div>',
				'aspect_ratio' => 0,
				'size' => '30x30'
			),
			'point-of-interest-white' => array(
				'html' => '<div class="poi-single-point poi-white"><a class="poi-img-replace" href="#0">More</a><div class="poi-more-info poi-top"></div></div>',
				'aspect_ratio' => 0,
				'size' => '30x30'
			),
			'point-of-interest-black' => array(
				'html' => '<div class="poi-single-point poi-black"><a class="poi-img-replace" href="#0">More</a><div class="poi-more-info poi-top"></div></div>',
				'aspect_ratio' => 0,
				'size' => '30x30'
			),

			'point-of-interest-red-round' => array(
				'html' => '<div class="poi-single-point poi-red _dn_round"><a class="poi-img-replace" href="#0">More</a><div class="poi-more-info poi-top"></div></div>',
				'aspect_ratio' => 0,
				'size' => '30x30'
			),
			'point-of-interest-blue-round' => array(
				'html' => '<div class="poi-single-point poi-blue _dn_round"><a class="poi-img-replace" href="#0">More</a><div class="poi-more-info poi-top"></div></div>',
				'aspect_ratio' => 0,
				'size' => '30x30'
			),
			'point-of-interest-white-round' => array(
				'html' => '<div class="poi-single-point poi-white _dn_round"><a class="poi-img-replace" href="#0">More</a><div class="poi-more-info poi-top"></div></div>',
				'aspect_ratio' => 0,
				'size' => '30x30'
			),
			'point-of-interest-black-round' => array(
				'html' => '<div class="poi-single-point poi-black _dn_round"><a class="poi-img-replace" href="#0">More</a><div class="poi-more-info poi-top"></div></div>',
				'aspect_ratio' => 0,
				'size' => '30x30'
			),

			'point-of-interest-round' => array(
				'html' => '<div class="poi-single-point _dn_round"><a class="poi-img-replace" href="#0">More</a><div class="poi-more-info poi-top"></div></div>',
				'aspect_ratio' => 0,
				'size' => '30x30'
			),
			'point-of-interest-square' => array(
				'html' => '<div class="poi-single-point"><a class="poi-img-replace" href="#0">More</a><div class="poi-more-info poi-top"></div></div>',
				'aspect_ratio' => 0,
				'size' => '30x30'
			),
		);

		return !empty($args['key']) ? $shapes[$args['key']] : $shapes;

	}




	/*
	 * HTML Elements
	 *
	 * @access public
	 * @return array
	*/
	public static function html_elements($args = array())
	{
		$defaults = array(
			'type' => 'html',
			'linid' => rand(),
			'key' => '', 
		);
		$args = ADN_Main::parse_args($args, $defaults);


		// HTML ELEMENTS
		$default_class = ' _dn_elem html_element element';
		$elements['html'] = array(
			'input-field' => array(
				'icon' => '<div class="ttip" title="'.__('Input Field','imc').'" style="background-color:#FFFFFF;height:100%;"><svg viewBox="0 0 363.334 297.332" style="width:30px;margin-top:9px;"><path stroke="#000000" stroke-width="6" stroke-miterlimit="10" d="M5.013,228.533"/><path stroke="#000000" stroke-width="6" stroke-miterlimit="10" d="M5.013,228.533"/><line stroke="#000000" stroke-width="6" stroke-miterlimit="10" x1="218" y1="5.156" x2="308.909" y2="5.156"/><line stroke="#000000" stroke-width="6" stroke-miterlimit="10" x1="263.455" y1="5.156" x2="263.455" y2="292.169"/><line stroke="#000000" stroke-width="6" stroke-miterlimit="10" x1="218" y1="292.169" x2="308.909" y2="292.169"/><path stroke="#000000" stroke-width="6" stroke-miterlimit="10" d="M82.935,118.143"/><path stroke="#000000" stroke-width="6" stroke-miterlimit="10" d="M82.935,118.143"/><line stroke="#000000" stroke-width="6" stroke-miterlimit="10" x1="59.727" y1="167.343" x2="103.391" y2="167.423"/><path fill="none" stroke="#000000" stroke-width="6" stroke-miterlimit="10" d="M5.013,228.533"/><polyline fill="none" stroke="#000000" stroke-width="6" stroke-miterlimit="10" points="229.688,228.533 5.013,228.533 5.013,75.286 229.688,75.286 "/><polyline fill="none" stroke="#000000" stroke-width="6" stroke-miterlimit="10" points="50.667,186.974 82.935,118.143 112,186.974 "/><polyline fill="none" stroke="#000000" stroke-width="6" stroke-miterlimit="10" points="298.52,228.533 359,228.533 359,75.286 359,75.286 298.52,75.286 "/></svg></div>',
				'html' => '<input type="text" class="input'.$default_class.'" style="width:100%;height:100%;" />',
				'params' => array(
					'placeholder' => array(
						'val' => '',
						'editing' => array('input' => array())
					)
				),
				'aspect_ratio' => 0,
				'size' => '200x40'
			),
		);

		// VIDEO ELEMENTS
		$default_class = ' _dn_elem video_element element';
		$elements['video'] = array(
			'youtube' => array(
				'icon' => '<div class="ttip" title="'.__('Youtube','imc').'" style="background-color:#FFFFFF;height:100%;"><svg viewBox="0,0,576,512" style="margin-top:2px;fill:#EB3223;"><path d="M549.655 124.083c-6.281-23.65-24.787-42.276-48.284-48.597C458.781 64 288 64 288 64S117.22 64 74.629 75.486c-23.497 6.322-42.003 24.947-48.284 48.597-11.412 42.867-11.412 132.305-11.412 132.305s0 89.438 11.412 132.305c6.281 23.65 24.787 41.5 48.284 47.821C117.22 448 288 448 288 448s170.78 0 213.371-11.486c23.497-6.321 42.003-24.171 48.284-47.821 11.412-42.867 11.412-132.305 11.412-132.305s0-89.438-11.412-132.305zm-317.51 213.508V175.185l142.739 81.205-142.739 81.201z"></path></svg></div>',
				'html' => '<iframe class="_dn_elem youtube" frameborder="0" allowfullscreen="1" allow="autoplay; encrypted-media" title="YouTube video player" width="100%" height="100%" src="https://www.youtube.com/embed/0"></iframe>',
				//'html' => ADN_Templates::youtube_player_api(array('settings' => array())),
				'params' => array(
					'video_id' => array(
						'val' => '',
						'editing' => array('input' => array())
					),
					'autoplay' => array(
						'val' => 0,
						'editing' => array('checkbox' => array('on' => 1, 'off' => 0))
					),
					'show_controls' => array(
						'val' => 1,
						'editing' => array('checkbox' => array('on' => 1, 'off' => 0))
					),
					'loop' => array(
						'val' => 0,
						'editing' => array('checkbox' => array('on' => 1, 'off' => 0))
					)
				),
				'aspect_ratio' => 0,
				'size' => '200x200'
			),
		);

		if( !empty($args['type']))
		{
			return !empty($args['key']) ? $elements[$args['type']][$args['key']] : $elements[$args['type']];
		}
		else
		{
			return $elements;
		}
	}



	




	/*
	 * Load GIPHY animated GIFs
	 *
	 * @access public
	 * @return array
	*/
	public static function load_giphy_gifs($args = array())
	{
		$settings = self::load_settings();
		$defaults = array(
			'action' => 'trending',
			'query' => '',
			'per_page' => 10
		);
		$args = ADN_Main::parse_args($args, $defaults);

		//$url = "http://api.giphy.com/v1/gifs/search?q=ryan+gosling&api_key=unKJddZJZinYYKrPtuILc2JN9rJcCby2&limit=10";
		//$url = "http://api.giphy.com/v1/stickers/search?q=lol&api_key=unKJddZJZinYYKrPtuILc2JN9rJcCby2&limit=10";
		//$url = "http://api.giphy.com/v1/stickers/trending?api_key=unKJddZJZinYYKrPtuILc2JN9rJcCby2&limit=10";
		//$gifs = json_decode(file_get_contents($url),true);
		//echo '<pre>'.print_r($gifs, true).'</pre>';

		$query = !empty($args['query']) ? '&q='.$args['query'] : '';
		$url = $settings['image']['stock_providers']['giphy']['url'].'stickers/'.$args['action'].'?api_key='.$settings['image']['stock_providers']['giphy']['application_id'].$query.'&limit='.$args['per_page'];
		//echo $url;
		if( !empty($settings['image']['stock_providers']['giphy']['application_id']))
		{
			$contents = json_decode(file_get_contents($url), true);
			//echo '<pre>'.print_r($contents, true).'</pre>';
			// If $contents is not a boolean FALSE value.
			if($contents !== false) 
			{
				return $contents;
			}
		}
	}







	/*
	 * Load Unsplash photos
	 *
	 * https://smpetrey.com/embed-unsplash-photos-wordpress/
	 * 
	 * PEXELS API: 563492ad6f917000010000013755de54ef4f4c9c80037a61055733bc
	 * 
	 * @access public
	 * @return array
	*/
	public static function load_unsplash_photos($args = array())
	{
		$settings = self::load_settings();
		$defaults = array(
			//'application_id' => '',
			//'url' => 'https://api.unsplash.com/',  // search/photos?page=1&query=office
			'action' => 'search',
			'query' => '',
			'per_page' => 10
		);
		$args = ADN_Main::parse_args($args, $defaults);

		$query = !empty($args['query']) ? '&query='.$args['query'] : '';
		$url = $settings['image']['stock_providers']['unsplash']['url'].$args['action'].'/photos?page=1&per_page='.$args['per_page'].$query.'&client_id='.$settings['image']['stock_providers']['unsplash']['application_id'];

		if( !empty($settings['image']['stock_providers']['unsplash']['application_id']))
		{
			$contents = json_decode(file_get_contents($url), true);
			// If $contents is not a boolean FALSE value.
			if($contents !== false) 
			{
				return $contents;
			}
		}
	}


	
	
	/*
	 * Common Banner Sizes
	 *
	 * @access public
	 * @return array
	*/
	public static function banner_sizes()
	{
		return array(
			array('size' => '125x125', 'name' => __('square button','imc')),
			array('size' => '300x250', 'name' => __('medium rectangle','imc')),
			array('size' => '728x90', 'name' => __('leaderboard','imc')),
			array('size' => '728x300', 'name' => __('pop-under','imc')),
			array('size' => '234x60', 'name' => __('half banner','imc')),
			array('size' => '468x60', 'name' => __('full banner','imc')),
			array('size' => '120x600', 'name' => __('skyscraper','imc')),
			array('size' => '160x600', 'name' => __('wide skyscraper','imc')),
			array('size' => '240x600', 'name' => __('extra wide skyscraper','imc')),
			array('size' => '300x600', 'name' => __('half page ad','imc')),
			array('size' => '120x240', 'name' => __('vertical banner','imc')),
			array('size' => '240x400', 'name' => __('vertical rectangle','imc'))
		);
	}
	
	
	
	
	



	public static function embed_args($args = array(), $ats = '')
	{
		$i = 0;
		foreach ($args as $key => $value) 
		{
			$coma = !$i ? '' : ',';
			$val = empty($value) ? 0 : $value;

			if(is_array($value))
			{
				$ats.= $coma.'"'.$key.'":{'.self::embed_args($value).'}';
				//return $ats;
			}
			else
			{
				$ats.= $coma.'"'.$key.'":"'.$val.'"';
			}
			$i++;
		}
		
		return $ats;	
	}
	
	
	
	public static function embed_code($args = array())
	{
		$defaults = array(
			'id' => 0,
			'animation' => '',
			'sa' => 'iv',
			'load_jquery' => 0,
			'use_iframe' => 0
		);
		$args = ADN_Main::parse_args($args, $defaults);
		
		//$bo = ADN_Banner::load_banner_obj($args['id']);
		$bo = '';
		$assets_url = parse_url(ADN_ASSETS_URL);

		$ats = '';
		if(!empty($args))
		{
			$embed_args = $args;
			// Remove some attribuetes we don't need for the banner itself.
			unset($embed_args['load_jquery']);
			unset($embed_args['use_iframe']);

			$ats = self::embed_args($args);
		}
		
		$url = get_site_url();
		$hash = self::getToken();
		$h = '';

		if(empty($args['use_iframe']))
		{
			// JAVASCRIPT EMBED CODE
			$h.= '<!--/** START ADNING **/-->';
			$h.= '<div id="'.$hash.'"></div>';
			// Load jQuery
			//$h.= $args['load_jquery'] ? '<script type=\"text/javascript\">var u=\"'.$url.'\";var a=\"'.$assets_url['path'].'\";if(!window.jQuery){var s=document.createElement(\"script\");s.type=\"text/javascript\";s.src=u+a+\"/js/embed/jquery.js\";document.getElementsByTagName(\"head\")[0].appendChild(s);}<\/script>' : '';
			$h.= $args['load_jquery'] ? '<script type="text/javascript">var u="'.$url.'";var a="'.$assets_url['path'].'";if(!window.jQuery){var s=document.createElement("script");s.type="text/javascript";s.src=u+a+"/js/embed/jquery.js";document.getElementsByTagName("head")[0].appendChild(s);}</script>' : '';

			$h.= '<script type="text/javascript" src="'.$url.'?_dnembed=true"></script>';
			$h.= '<script type="text/javascript">jQuery("#'.$hash.'")._dn_frame({'.$ats.'});</script>';
			$h.= '<!--/** END ADNING */-->';
		}
		else
		{
			// IFRAME EMBED CODE
			if($args['responsive'])
			{
				$h.= '<div style="max-width:'.$args['width'].'px; width:100%; height:'.$args['height'].'px;"><iframe src="'.$url.'?_dnid='.$args['id'].'" border="0" scrolling="no" allowtransparency="true" style="width:1px;min-width:100%;*width:100%;height:100%;border:0;"></iframe></div>'; //width="100%" height="100%"
			}
			else
			{
				$h.= '<iframe src="'.$url.'?_dnid='.$args['id'].'" border="0" scrolling="no" allowtransparency="true" width="'.$args['width'].'px" height="'.$args['height'].'px" style="border:0;"></iframe>';
			}
		}
		
		return $h;
	}
	
	
	
	
	/**
	 * Create Unique Token/Hash
	 * https://stackoverflow.com/a/13733588/3481803
	*/
	public static function getToken($length = 6)
	{
		$token = "";
		$codeAlphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
		$codeAlphabet.= "abcdefghijklmnopqrstuvwxyz";
		$codeAlphabet.= "0123456789";
		$max = strlen($codeAlphabet); // edited
	
		for ($i=0; $i < $length; $i++) {
			$token .= $codeAlphabet[self::crypto_rand_secure(0, $max-1)];
		}
	
		return $token;
	}
	public static function crypto_rand_secure($min, $max)
	{
		$range = $max - $min;
		if ($range < 1) return $min; // not so random...
		$log = ceil(log($range, 2));
		$bytes = (int) ($log / 8) + 1; // length in bytes
		$bits = (int) $log + 1; // length in bits
		$filter = (int) (1 << $bits) - 1; // set all lower bits to 1
		do {
			$rnd = hexdec(bin2hex(openssl_random_pseudo_bytes($bytes)));
			$rnd = $rnd & $filter; // discard irrelevant bits
		} while ($rnd > $range);
		return $min + $rnd;
	}




	/**
	 * Create data-* attributes
	 * data-key="value"
	*/
	public static function create_data_attributes($args = array())
	{
		$h = '';
		if(!empty($args))
		{
			foreach ($args as $key => $value) 
			{
				$h.= ' data-'.$key.'="'.$value.'"';
			}
		}

		return $h;
	}




	/**
	 * Load Banner Object
	*/
	/*public static function load_banner_obj( $id = 0)
	{
		if($id)
		{
			// Load banner data
			$banner_arr = ADN_DB::get_results(array(
				'sql' => 'SELECT * FROM '.ADN_DB::$wpdb->prefix.'adning_banners WHERE id = '.$id.' LIMIT 1'
			));
			$slides_arr = ADN_DB::get_results(array(
				'sql' => 'SELECT * FROM '.ADN_DB::$wpdb->prefix.'adning_slides WHERE id = '.$id
			));

			$slides = array();
			if(!empty($slides_arr))
			{
				foreach($slides_arr as $slide)
				{
					$slides[] = ADN_Main::parse_args(json_decode($slide['params'], true), array('slide_id' => $slide['slide_id'], 'slide_order' => $slide['slide_order'], 'layers' => json_decode($slide['layers'], true)));
				}
			}

			return ADN_Main::parse_args(json_decode($banner_arr[0]['settings'],true), array('slides' => $slides));
		}

		return json_decode('[]');
	}*/





	/**
	 * NOT USED
	 * Get banner status
	*/
	public static function banner_status( $id = 0)
	{
		
		return 0;
	}






	/**
	 * Example string:
	 * 		type|popup,attr:{close_btn:{delay:2000}}
	 *
	 * Returns
	 *		type:"popup",attr:{close_btn:{delay:2000}}
	 *
	*/
	public static function create_object_from_shortcode($string){
		$parts = explode(',', $string);
		$object = '';
		if( !empty($parts))
		{
			foreach($parts as $i => $part){
				$coma = !$i ? '' : ',';
				$itms = explode('|', $part);

				if(count($itms) > 1){
					$str = $coma.$itms[0].':"'.$itms[1].'"';
					$object.= $str;
				}else{
					$str = $coma.$part;
					$object.= $str;
				}
				
			}
		}
		//echo $object;
		return $object;
	}





	/**
	 * Get Folder/file size in bytes
	*/
	public static function getSize($path)
	{
	    $size = 0;
	    if(is_file($path))
	    	return filesize( $path );

	    foreach (glob(rtrim($path, '/').'/*', GLOB_NOSORT) as $each) {
	        $size += is_file($each) ? filesize($each) : self::getSize($each);
	    }
	    return $size;
	}



	/**
	 * Format bytes
	*/
	public static function formatBytes($size, $precision = 0)
	{
	    $unit = array('B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'); 

	    for($i = 0; $size >= 1024 && $i < count($unit)-1; $i++){
	        $size /= 1024;
	    }

	    return round($size, $precision).' '.$unit[$i];
	}






	/**
	 * File get contents
	*/
	public static function get_content($args = array())
	{
		$defaults = array(
			'path' => '',
			'spacefix' => 0 // Used for some issues with .svg files 
		);
		$args = self::parse_args($args, $defaults);
		$content = ''; //sprintf(__('Could not load file %s','imc'), $args['path'])

		if( !empty($args['path']) && file_exists($args['path']))
		{
			$content = file_get_contents($args['path']);

			if( $args['spacefix'] ){
				$content = self::spacefix($content);
			}
		}

		return $content;
	}



	/**
	 * Spacefix
	 * 
	 * Used for some issues with .svg files 
	 */
	public static function spacefix($content = '')
	{
		$content = str_replace(array("\r\n","\r","\n","\\r","\\n","\\r\\n"),"",$content);
		$content = str_replace(array("\t","\\t")," ",$content);

		return $content;
	}





	/**
	 *  LOAD TEMPLATES
	 */
	public static function library_template_files($dir)
	{
		if(!file_exists($dir)) return;
		$ffs = scandir($dir);
		$templates = array();

		unset($ffs[array_search('.', $ffs, true)]);
		unset($ffs[array_search('..', $ffs, true)]);

		// prevent empty ordered elements
		if (count($ffs) < 1)
			return;
		
		foreach($ffs as $ff)
		{
			if (preg_match("/^[^\.].*$/", $ff)) 
			{
				if(is_dir($dir.'/'.$ff))
				{
					$templates[$ff] = self::library_template_files($dir.'/'.$ff);
				}

				// read json content
				if( !is_dir($dir.'/'.$ff))
				{
					/*$json_file = file_get_contents($dir.'/'.$ff);
					$json_file = str_replace(array("\r\n","\r","\n","\\r","\\n","\\r\\n"),"",$json_file);
					$json_file = str_replace(array("\t","\\t")," ",$json_file);*/
					$json_file = self::get_content(array('path' => $dir.'/'.$ff, 'spacefix' => 1));
					
					$banner_obj = json_decode(stripslashes($json_file),true);
					// fix, in some cases stripslashes does not work.
					$banner_obj = empty($banner_obj) ? json_decode($json_file,true) : $banner_obj;

					$templates[] = array(
						'filename' => basename($ff),
						'json' => $banner_obj
					);
				}
			}
		}
		return $templates;
	}



	/**
	 *  LOAD SVG STICKERS
	 */
	public static function library_sticker_files($dir)
	{
		if(!file_exists($dir)) return;

		$ffs = scandir($dir);
		$stickers = array();

		unset($ffs[array_search('.', $ffs, true)]);
		unset($ffs[array_search('..', $ffs, true)]);

		// prevent empty ordered elements
		if (count($ffs) < 1)
			return;

		foreach($ffs as $ff)
		{
			if (preg_match("/^[^\.].*$/", $ff)) 
			{
				if(is_dir($dir.'/'.$ff))
				{
					$stickers[$ff] = self::library_sticker_files($dir.'/'.$ff);
				}

				// read svg content
				if( !is_dir($dir.'/'.$ff))
				{
					// Only load SVG files.
					$path_info = pathinfo($dir.'/'.$ff);
					if( $path_info['extension'] === 'svg' )
					{
						$svg_file = file_get_contents($dir.'/'.$ff);
						$stickers[] = self::svg_builder(array('code' => $svg_file, basename($ff)));
					}
				}
			}
		}

		unset($stickers['__MACOSX']);
		return $stickers;
	}






	public static function svg_builder($args = array(), $filename = '')
	{
		$defaults = array(
			'code' => ''
		);
		$args = self::parse_args($args, $defaults);

		$svg = '';
		$svg_file = $args['code'];

		if( !empty($svg_file))
		{
			$animated = 0;
			$animation_option = 0;
			$matches = array();
			$vbmatches = array();
			preg_match('/<svg (.*?)>(.*?)<\/svg>/s', $svg_file, $matches, PREG_OFFSET_CAPTURE);
			preg_match('/viewBox="(.*?)"/s', $matches[1][0], $vbmatches, PREG_OFFSET_CAPTURE);
			preg_match('/class="(.*?)"/s', $matches[1][0], $csmatches, PREG_OFFSET_CAPTURE);

			$svg_content = $matches[2][0];
			$viewbox = !empty($vbmatches[0][0]) ? $vbmatches[0][0] : '';
			$class = !empty($csmatches[0][0]) ? $csmatches[0][0] : '';
			$data = array();
			//echo '<pre>'.print_r($stroke_matches, true).'</pre>';
			
			// Check if svg has animation
			$animated = !empty($csmatches[1][0]) ? strpos($csmatches[1][0], '_dn_ani') !== false ? 1 : 0 : 0;
			if( !$animated )
			{
				// Check if strokes are found as they can be animated as an option later.
				preg_match('/<path (.*?)>/s', $svg_file, $pmatches, PREG_OFFSET_CAPTURE);
				if( !empty($pmatches) ){
					preg_match('/stroke="(.*?)"/s', $pmatches[1][0], $stroke_matches, PREG_OFFSET_CAPTURE);
					// Found a stroke
					$animation_option = !empty($stroke_matches) ? 1 : 0;
				}
			}
			
			// in case CSS STYLE FILL
			$matches = array();
			preg_match_all('/style=\"fill:(.*?);\"/s', $svg_content, $matches, PREG_OFFSET_CAPTURE);

			$c = 1;
			$fills = array();
			foreach ($matches[0] as $key => $value) 
			{
				if (false !== $arr_key = array_search($matches[1][$key][0], $fills)) 
				{
					$svg_content = str_replace($value[0], 'style="fill:'.$matches[1][$key][0].';" class="colorful c'.($arr_key+1).'"', $svg_content);
				}
				else
				{
					$svg_content = str_replace($value[0], 'style="fill:'.$matches[1][$key][0].';" class="colorful c'.$c.'"', $svg_content);
				}

				if( !in_array($matches[1][$key][0], $fills) )
				{
					$fills[] = $matches[1][$key][0];
					$c++;
				}
			}
			//print_r($fills);

			// in case SVG FILL
			$matches = array();
			preg_match_all('/fill=\"(.*?)\"/s', $svg_content, $matches, PREG_OFFSET_CAPTURE);

			$c = 1;
			$fills = array();
			foreach ($matches[0] as $key => $value) 
			{
				if( strpos($matches[1][$key][0], 'url') === false ){
					//$svg_content = str_replace($value[0], 'fill="'.$matches[1][$key][0].'" class="colorful c'.$c.'"', $svg_content);
					if (false !== $arr_key = array_search($matches[1][$key][0], $fills)) 
					{
						$svg_content = str_replace($value[0], 'fill="'.$matches[1][$key][0].'" class="colorful c'.($arr_key+1).'"', $svg_content);
					}
					else
					{
						$svg_content = str_replace($value[0], 'fill="'.$matches[1][$key][0].'" class="colorful c'.$c.'"', $svg_content);
					}

					if( !in_array($matches[1][$key][0], $fills) )
					{
						$fills[] = $matches[1][$key][0];
						$c++;
					}
				}
			}

			// UNIQUE IDs
			$matches = array();
			preg_match_all('/id=\"(.*?)\"/s', $svg_content, $matches, PREG_OFFSET_CAPTURE);
			//echo '<pre>'.print_r($matches, true).'</pre>';
			foreach ($matches[1] as $key => $value) 
			{
				$new_id = time().'_'.rand();
				$svg_content = str_replace($value[0], $new_id, $svg_content);
			}


			// Create data-*
			$data_str = '';
			if( !empty($data))
			{
				foreach($data as $data_key => $dt)
				{
					$data_str.= ' data-'.$data_key.'="'.$dt.'"';
				}
			}

			$svg = array(
				'vb' => $viewbox,
				'animation' => $animated,
				'animation_option' => $animation_option,
				'filename' => $filename,
				'svg' => '<svg '.$class.' '.$viewbox.$data_str.'>'.$svg_content.'</svg>'
			);
		}
		
		return $svg;
	}




	public static function str_replace($args = array())
	{
		$defaults = array(
			'replace' => array(),
			'string' => ''
		);
		$args = self::parse_args($args, $defaults);
		$str = $args['string'];

		if( !empty($str) && !empty($args['replace']))
		{
			foreach($args['replace'] as $_replace)
			{
				if( !empty($_replace))
				{
					foreach($_replace as $search => $replace)
					{
						$str = str_replace($search, $replace, $str);
					}
				}
			}
		}

		return $str;
	}




	public static function create_url_from_structure($structure = '', $args = array())
	{
		$defaults = array(
			'id' => '',
			'uid' => ImgMCE::$current_user_id,
			'dir' => IMC_UPLOAD_DIR,
			'folder' => ''
		);
		$args = self::parse_args($args, $defaults);

		$dir = rtrim($args['dir'], '/');
		$folder = rtrim($args['folder'], '/');
		
		return ADN_Main::str_replace(array(
			'replace' => array(
				array('{ID}' => $args['id']),
				array('{DIR}' => $dir),
				array('{FOLDER}' => $folder),
				array('{UID}' => $args['uid'])
			), 
			'string' => $structure
		));
	}





	/** 
	 * Merge multidimentional arrays.
	 * https://gist.github.com/ptz0n/1646171
	 * 
	 * ADN_Main::array_merge_recursive_distinct( $defaults, $args );
	*/
	/*public static function array_merge_recursive_distinct()
	{
		$arrays = func_get_args();
		$base = array_shift($arrays);
		foreach($arrays as $array) {
			reset($base);
			while(list($key, $value) = @each($array)) {
				if(is_array($value) && @is_array($base[$key])) {
					$base[$key] = self::array_merge_recursive_distinct($base[$key], $value);
				}
				else {
					$base[$key] = $value;
				}
			}
		}
		return $base;
	}*/
	/** 
	 * Merge multidimentional arrays.
	 * http://php.net/manual/en/function.array-merge-recursive.php#102379
	 * 
	 * ADN_Main::array_merge_recursive_distinct( $defaults, $args );
	*/
	public static function array_merge_recursive_distinct( $defaults, $args )
	{
		if( !is_array($defaults) )
			return $args;
		
		foreach($args as $key => $Value)
		{
			if(array_key_exists($key, $defaults) && is_array($Value))
			{
				$defaults[$key] = self::array_merge_recursive_distinct($defaults[$key], $args[$key]);
			}
			else
			{
				$defaults[$key] = $Value;
			}
		}

		return $defaults;
	}



	/**
	 * Return correct file folder/name based on debug info
	 */
	public static function file_loc($args = array())
	{
		$defaults = array(
			'debug' => IMC_DEBUG,
			'name' => ''
		);
		$args = ADN_Main::parse_args($args, $defaults);

		$fld = $args['debug'] ? '_dev/' : 'dist/';
		$ext = $args['debug'] ? '' : '.min';

		return $fld.$args['name'].$ext;
	}
	


	// RATIO
	public static function ratio($a, $b)
	{
		$gcd = function($a, $b) use (&$gcd) {
			return ($a % $b) ? $gcd($b, $a % $b) : $b;
		};
		$g = $gcd($a, $b);
		return $a/$g . ':' . $b/$g;
	}





	public static function delete_dir($dir) 
	{
		$res = array('removed' => 0, 'msg' => __('Folder could not be removed. Please try again.','imc'));

		if (! is_dir($dir)) {
			//throw new InvalidArgumentException("$dir must be a directory");
			return array('removed' => 0, 'msg' => sprintf(__('Error: %s is no directory.','imc'), $dir));
		}
		if(! is_writable($dir)) {
			return array('removed' => 0, 'msg' => sprintf(__('Error: permission denied. %s','imc'), $dir));
		}
		if (substr($dir, strlen($dir) - 1, 1) != '/') {
			$dirPath .= '/';
		}
		$files = glob($dir . '*', GLOB_MARK);
		foreach ($files as $file) {
			if (is_dir($file)) {
				self::delete_dir($file);
			} else {
				unlink($file);
			}
		}
		rmdir($dir);

		return array('removed' => 1, 'msg' => __('Image removed successfully.','imc'));
	}


	// ZIP FOLDER
	// NOT LONGER USED AS IT DOESN'T INCLUDE THE PARENT FOLDER
	// https://stackoverflow.com/questions/1334613/how-to-recursively-zip-a-directory-in-php
	/*public static function Zip($args = array())
	{
		$defaults = array(
			'id' => 0,
			'upload' => array(
				'dir' => '',
				'src' => '',
				'folder' => ''
			),
			'delete_zips_in_folder' => 0
		);
		$args = ADN_Main::parse_args($args, $defaults);

		$id = $args['id'];
		$path = $args['upload']['dir'].$args['upload']['folder'].$id;
		$path = rtrim($path, '/');

		if (!extension_loaded('zip') || !file_exists($path)) {
			return array('zipped' => 0, 'msg' => sprintf(__('Error: zip extention not loaded or file %s does not exists.','imc'), $path));
		}

		// Create image folder if not exists. This fixes the parent folder from being removed when unzipping.
		if(!is_dir($path.'/images/')) {
		    mkdir($path.'/images/', 0777, true);
		}

		$zip = new ZipArchive();
		if (!$zip->open($path.'.zip', ZIPARCHIVE::CREATE)) {
			return array('zipped' => 0, 'msg' => sprintf(__('Error: could not zip folder to %s.','imc'), $path.'.zip'));
		}


		// Remove all existing .zip files
		// This is not necessary but we don't want to save them in that folder.
		if( $args['delete_zips_in_folder'] )
		{
			$files = glob($args['upload']['dir'].$args['upload']['folder'].'*.zip'); // get all file names
			foreach($files as $file){
				if(is_file($file)){
					unlink($file); // delete file
				}
			}
		}


		$path = str_replace('\\', '/', realpath($path));

		if (is_dir($path) === true)
		{
			$files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($path), RecursiveIteratorIterator::SELF_FIRST);

			foreach ($files as $file)
			{
				$file = str_replace('\\', '/', $file);

				// Ignore "." and ".." folders
				if( in_array(substr($file, strrpos($file, '/')+1), array('.', '..')) )
					continue;

				$file = realpath($file);

				if (is_dir($file) === true)
				{
					$zip->addEmptyDir(str_replace($path . '/', '', $file . '/'));
				}
				else if (is_file($file) === true)
				{
					$zip->addFromString(str_replace($path . '/', '', $file), file_get_contents($file));
				}
			}
		}
		else if (is_file($dir) === true)
		{
			$zip->addFromString(basename($path), file_get_contents($path));
		}

		$zip->close();

		return array('zipped' => 1, 'src' => $args['upload']['src'].$args['upload']['folder'].$id.'.zip', 'msg' => sprintf(__('Folder %s is zipped.','imc'), $path.'.zip'));
	}*/



	





	/** 
	 * Add files and sub-directories in a folder to zip file. 
	 * @param string $folder 
	 * @param ZipArchive $zipFile 
	 * @param int $exclusiveLength Number of text to be exclusived from the file path. 
	 */ 
	private static function folderToZip($folder, &$zipFile, $exclusiveLength) { 
		$handle = opendir($folder); 
		while (false !== $f = readdir($handle)) { 
		if ($f != '.' && $f != '..') { 
			$filePath = "$folder/$f"; 
			// Remove prefix from file path before add to zip. 
			$localPath = substr($filePath, $exclusiveLength); 
			if (is_file($filePath)) { 
			$zipFile->addFile($filePath, $localPath); 
			} elseif (is_dir($filePath)) { 
			// Add sub-directory. 
			$zipFile->addEmptyDir($localPath); 
			self::folderToZip($filePath, $zipFile, $exclusiveLength); 
			} 
		} 
		} 
		closedir($handle); 
	} 

	/** 
	 * Zip a folder (including itself).
	 *
	 */ 
	public static function zipDir($args = array()) 
	{ 
		$defaults = array(
			'id' => 0,
			'upload' => array(
				'dir' => '',
				'src' => '',
				'folder' => ''
			),
			'delete_zips_in_folder' => 0
		);
		$args = ADN_Main::parse_args($args, $defaults);

		$id = $args['id'];
		$path = $args['upload']['dir'].$args['upload']['folder'].$id;
		$path = rtrim($path, '/');

		if (!extension_loaded('zip') || !file_exists($path)) {
			return array('zipped' => 0, 'msg' => sprintf(__('Error: zip extention not loaded or file %s does not exists.','imc'), $path));
		}

		// Remove all existing .zip files
		// This is not necessary but we don't want to save them in that folder.
		if( $args['delete_zips_in_folder'] )
		{
			$files = glob($args['upload']['dir'].$args['upload']['folder'].'*.zip'); // get all file names
			foreach($files as $file){
				if(is_file($file)){
					unlink($file); // delete file
				}
			}
		}

		$pathInfo = pathInfo($path); 
		$parentPath = $pathInfo['dirname']; 
		$dirName = $pathInfo['basename']; 

		$z = new ZipArchive();
		if (!$z->open($path.'.zip', ZIPARCHIVE::CREATE)) {
			return array('zipped' => 0, 'msg' => sprintf(__('Error: could not zip folder to %s.','imc'), $path.'.zip'));
		}
		//$z = new ZipArchive(); 
		//$z->open($path.'.zip', ZIPARCHIVE::CREATE); 
		$z->addEmptyDir($dirName); 
		self::folderToZip($path, $z, strlen("$parentPath/")); 
		$z->close(); 

		return array('zipped' => 1, 'src' => $args['upload']['src'].$args['upload']['folder'].$id.'.zip', 'path' => $path.'.zip', 'msg' => sprintf(__('Folder %s is zipped.','imc'), $path.'.zip'));
	} 





	/**
	 * Load Layer Plugins JS
	 */
	public static function layer_plugins($args = array())
	{
		$defaults = array(
			'layer' => array(),
			'hash' => '',
			'key' => 0,
			'group' => '',
			'animation' => false // load animation plugins only. If "false" animation plugins will be ignored.
		);
		$args = self::parse_args($args, $defaults);

		// Layer Plugins (effects)
		$layer = $args['layer'];
		$hash = $args['hash'];
		$key = $args['key'];
		$group = $args['group'];

		$js = '';
		$external = '';
		$external_scripts = array('css' => array(), 'js' => array());
		$am_external_scripts = 0;
		$plugins_code = '';

		if( array_key_exists('plugins', $layer))
		{
			if( !empty($layer['plugins']))
			{
				// Hide layer till all necessary scripts are loaded.
				$js.= '$("._dn_'.$hash.'").find(".layer_'.$key.'").css({"opacity":0});';

				foreach($layer['plugins'] as $pl => $plugin)
				{
					if( array_key_exists('type', $plugin))
					{
						$load_plugin = $plugin['type'] == 'animation' && !$args['animation'] ? 0 : 1;
						$load_plugin = $plugin['type'] != 'animation' && $args['animation'] ? 0 : $load_plugin;
					}
					else
					{
						$load_plugin = 1;
					}

					if( $load_plugin ){
						foreach ($plugin['external'] as $ext => $types)
						{
							if(!empty($types))
							{
								foreach($types as $type){
									if(!empty($type))
									{
										if( is_array($type) && !empty($type['path'])){
											$url = $type['path']['dir'].$type['path']['folder'].$type['path']['file'];
										}else{
											$url = $type;
										}
										//echo $pl.' '.$ext.' '.$url.'<br>';
										$am_external_scripts++;
										$external_scripts[$ext][] = $url;

										// AMP - Add Google Fonts to head
										if( ADN_AMP::is_amp() )
										{							
											if( function_exists( 'is_amp_endpoint' ) ) 
											{
												//echo '<pre>'.print_r($layer['plugins']['google_font'],true).'</pre>';
												//$layer['plugins']['google_font']['external']['css'][0]
												if( $pl === 'google_font')
												{
													$font_key = sanitize_key($plugin['font']);
													$GLOBALS['font_urls'][$font_key] = sanitize_url($url);
													
													add_action( 'amp_post_template_data', function($data){ return imc_add_a_font($data); } );

													if( !function_exists('imc_add_a_font')){
														function imc_add_a_font($data){
															$data['font_urls'] = wp_parse_args($GLOBALS['font_urls'], $data['font_urls']);
															return $data;
														}
													}
												}
											}
										}
										// End AMP
									}
								}
							}
						}

						
						// Add group key if not exists
						if( !array_key_exists('group', $plugin)) { $plugin['group'] = ''; }
						
						if(array_key_exists('code', $plugin))
						{
							if( !empty($plugin['code']['run']))
							{
								if(empty($group))
								{
									$plugins_code.= self::str_replace(array(
										'replace' => array(
											array('{LAYER}' => '$("._dn_'.$hash.'").find(".layer_'.$key.'")'),
											array('{LAYER-CLASS}' => $layer['class']),
											array('{CLASS}' => array_key_exists('class', $plugin['params']) ? $plugin['params']['class'] : ''),
											array('{PARAMS}' => array_key_exists('params', $plugin) ? json_encode($plugin['params']) : '')
										),
										'string' => $plugin['code']['run']
									));
								}
								else
								{
									//$js.= 'console.log("'.$group.'");';
									//$js.= 'console.log('.json_encode($plugin['group']).');';
									if(!empty($plugin['group'][$group]))
									{
										$plugins_code.= self::str_replace(array(
											'replace' => array(
												array('{LAYER}' => '$("._dn_'.$hash.'").find(".layer_'.$key.'")'),
												array('{LAYER-CLASS}' => $layer['class']),
												array('{CLASS}' => array_key_exists('class', $plugin['group'][$group]) ? $plugin['group'][$group]['class'] : ''),
												array('{PARAMS}' => array_key_exists('group', $plugin) ? json_encode($plugin['group'][$group]) : '')
											),
											'string' => $plugin['code']['run']
										));
									}
								}
							}
						}
					}
				}

				if( $am_external_scripts )
				{
					/*foreach($external_scripts['css'] as $css)
					{
						wp_enqueue_style('kak_'.sanitize_key($css), $css);
						//add_action( 'wp_footer', function(){ echo 'kak'; }, PHP_INT_MAX );
					}
					//echo '<pre>'.print_r($external_scripts,true).'</pre>';
					*/

					// To preload scripts
					$external.= '_dn_get_scripts('.json_encode($external_scripts).');';
					
					$js.= !empty($plugin['group']) ? '/* GROUP */' : '';
					$js.= '_dn_get_scripts('.json_encode($external_scripts).', function () {';
						//$js.= 'console.log("ALL is loaded for layer_'.$key.'");';
						$js.= '$("._dn_'.$hash.'").find(".layer_'.$key.'").css({"opacity":1});';
						$js.= $plugins_code;
					$js.= '});';
				}
				else
				{
					$js.= '$("._dn_'.$hash.'").find(".layer_'.$key.'").css({"opacity":1});';
					$js.= $plugins_code;
				}
			}
		}

		return array('js' => $js, 'external' => $external);
	}





	/**
	 * LOAD ELEMENT TEMPLATE
	*/
	public static function load_element($args = array())
	{
		$defaults = array(
			'id' => 0,
			'user_id' => 0,
			'iframe_id' => 0,
			'add_url' => 0,
			'load_banner_script' => 1,
			'adjust_id' => '',
			'load_external' => 1,
			'animation' => 1,
			'start_animation' => 'iv',
			'display' => 0
		);
		$args = self::parse_args($args, $defaults);
		
		// *Important, Set decoding right
		ADN_Settings::magic_quotes();
		
		// Load banner_obj json directly from user_uploads folder 
		if( !empty($args['id']) ) // && !empty($args['user_id'])
		{
			$file = IMC_UPLOAD_DIR.'user_uploads/{UID}/{ID}/imgmce.json';
			$file = self::str_replace(array(
                'replace' => array(
                    array('{UID}' => $args['user_id']), 
                    array('{ID}' => $args['id']), 
                ), 
                'string' => $file
			));
			if( file_exists($file) )
			{
				$json = self::get_content(array('path' => $file));
				$banner_obj = json_decode(stripslashes($json), true);
				$banner_obj = empty($banner_obj) ? json_decode($json, true) : $banner_obj;
				//echo '<pre>'.print_r($banner_obj,true).'</pre>';

				$banner_template = ADN_Templates::banner_tpl($banner_obj, $args);
			}
			else
			{
				// Element does not exists.
				$banner_obj = array();
				$banner_template = '';
			}
		
			return array('banner_obj' => $banner_obj, 'id' => $args['id'], 'template' => $banner_template ); // 'timeline' => $timeline_template
		}
	}



	/**
	 * Return the duration of a specific slide.
	 */
	public static function slide_duration($slide = array())
	{
		$layers_with_animation = 0;
		$layer_timing = array();

		if(array_key_exists('layers', $slide) && !empty($slide['layers']))
		{
			foreach ($slide['layers'] as $nr => $layer) 
			{
				if(!empty($layer['animation']))
				{
					$layer_timing[$nr] = 0;
					$layers_with_animation++;
					foreach ($layer['animation'] as $ani => $group) 
					{
						$delay = array_key_exists('delay', $group) ? $group['delay'] : 0;
						$layer_timing[$nr] = $layer_timing[$nr]+$delay;
					}
				}
			}
		}

		arsort($layer_timing);
		//echo '<pre>'.print_r($layer_timing).'</pre>';

		return $layer_timing;
	}
	
}

endif;
?>