<?php
class ADN_Scriptloader {

    public $registered = array();
	public $queue = array();
	public $args = array();


    public function localize( $handle, $ext, $object_name, $l10n ) {
        if ( $handle === 'jquery' )
            $handle = 'jquery-core';
        

        if ( is_array($l10n) && isset($l10n['l10n_print_after']) ) { // back compat, preserve the code in 'l10n_print_after' if present
            $after = $l10n['l10n_print_after'];
            unset($l10n['l10n_print_after']);
        }

        foreach ( (array) $l10n as $key => $value ) {
            if ( !is_scalar($value) )
                continue;

            $l10n[$key] = html_entity_decode( (string) $value, ENT_QUOTES, 'UTF-8');
        }

		
		$json = $l10n;
        $script = "var $object_name = " . json_encode( $json ) . ';';
       
        $this->add_data( $handle, $ext, 'json', $json );


        if ( !empty($after) )
            $script .= "\n$after;";

        $data = $this->get_data( $handle, $ext, 'data' );


        if ( !empty( $data ) )
            $script = "$data\n$script";

        return $this->add_data( $handle, $ext, 'data', $script );
    }



    public function add( $handle, $src, $ext, $deps = array(), $ver = false, $args = null ) {
		if ( isset($this->registered[$ext][$handle]) )
			return false;
		
		$this->registered[$ext][$handle] = new _IMC_Dependency( $handle, $src, $deps, $ver, $args );
		return true;
    }
    

    public function enqueue( $handles, $ext ) {
		foreach ( (array) $handles as $handle ) {
			$handle = explode('?', $handle);
			if ( !in_array($handle[0], $this->queue) && isset($this->registered[$ext][$handle[0]]) ) {
				$this->queue[$ext][] = $handle[0];
				
				if ( isset($handle[1]) )
					$this->registered[$ext][$handle[0]]->args = $handle[1];
			}
		}
	}


    public function add_data( $handle, $ext, $key, $value ) {
        if ( !isset( $this->registered[$ext][$handle] ) )
            return false;

        return $this->registered[$ext][$handle]->add_data( $key, $value );
    }

    public function get_data( $handle, $ext, $key ) {
        if ( !isset( $this->registered[$ext][$handle] ) )
            return false;

        if ( !isset( $this->registered[$ext][$handle]->extra[$key] ) )
            return false;

        return $this->registered[$ext][$handle]->extra[$key];
    }





    public function register_script( $handle, $src, $deps = array(), $ver = false, $in_footer = false ) {

		$ext = pathinfo($src, PATHINFO_EXTENSION);

        $registered = $this->add( $handle, $src, $ext, $deps, $ver );
        if ( $in_footer ) {
            $this->add_data( $handle, $ext, 'group', 1 );
        }
    
        return $registered;
    }



    public function enqueue_script( $handle, $src = '', $deps = array(), $ver = false, $in_footer = false ) {
		$ext = '';
		
        if ( $src || $in_footer ) {
            $_handle = explode( '?', $handle );
    
            if ( $src ) {
				$ext = pathinfo($src, PATHINFO_EXTENSION);
                $this->add( $_handle[0], $src, $ext, $deps, $ver );
            }
    
            if ( $in_footer ) {
                $this->add_data( $_handle[0], $ext, 'group', 1 );
            }
		}

		$ext = !empty($ext) ? $ext : array_key_exists($handle, $this->registered['js']) ? 'js' : 'css';
        $this->enqueue( $handle, $ext );
	}
	



	public function print_scripts($handle = '')
	{
		$code = '';

		if(empty($handle))
		{
			foreach($this->queue as $ext => $handles)
			{
				foreach($handles as $handle)
				{
					$script = array_key_exists($handle, $this->registered[$ext]) ? $this->registered[$ext][$handle] : array();

					if( !empty($script))
					{
						$ver = !empty($script->ver) ? '?ver='.$script->ver : '';

						if( $ext == 'js')
						{
							$extra = !empty($script->extra) ? $script->extra : array();
							if(array_key_exists('data', $extra))
							{
								$code.= "\n<script type='text/javascript'>\n";
								$code.= "/* <![CDATA[ */\n"; // not needed in HTML 5
								$code.= $extra['data'];
								$code.= "/* ]]> */\n";
								$code.= "</script>\n";
							}
							$code.= sprintf( "<script type='text/javascript' src='%s'></script>\n", $script->src.$ver );
						}
						else
						{
							$code.= sprintf( "<link rel='stylesheet' href='%s' type='text/css' media='all' />\n", $script->src.$ver );
						}
					}
				}
			}
		}
		
		return $code;
	}


    /*
    public function localize( $handle, $object_name, $l10n ) {
        if ( $handle === 'jquery' )
            $handle = 'jquery-core';
        

        if ( is_array($l10n) && isset($l10n['l10n_print_after']) ) { // back compat, preserve the code in 'l10n_print_after' if present
            $after = $l10n['l10n_print_after'];
            unset($l10n['l10n_print_after']);
        }

        foreach ( (array) $l10n as $key => $value ) {
            if ( !is_scalar($value) )
                continue;

            $l10n[$key] = html_entity_decode( (string) $value, ENT_QUOTES, 'UTF-8');
        }

        $json = $l10n;
        $script = "var $object_name = " . json_encode( $json ) . ';';
       
        $this->add_data( $handle, 'json', $json );

        if ( !empty($after) )
            $script .= "\n$after;";

        
        $data = $this->get_data( $handle, 'data' );
        if ( !empty( $data ) )
            $script = "$data\n$script";

        return $this->add_data( $handle, 'data', $script );
    }



    public function add( $handle, $src, $deps = array(), $ver = false, $args = null ) {
		if ( isset($this->registered[$handle]) )
			return false;
		$this->registered[$handle] = new _IMC_Dependency( $handle, $src, $deps, $ver, $args );
		return true;
    }
    

    public function enqueue( $handles ) {
		foreach ( (array) $handles as $handle ) {
			$handle = explode('?', $handle);
			if ( !in_array($handle[0], $this->queue) && isset($this->registered[$handle[0]]) ) {
				$this->queue[] = $handle[0];
				
				if ( isset($handle[1]) )
					$this->registered[$handle[0]]->args = $handle[1];
			}
		}
	}


    public function add_data( $handle, $key, $value ) {
        if ( !isset( $this->registered[$handle] ) )
            return false;

        return $this->registered[$handle]->add_data( $key, $value );
    }

    public function get_data( $handle, $key ) {
        if ( !isset( $this->registered[$handle] ) )
            return false;

        if ( !isset( $this->registered[$handle]->extra[$key] ) )
            return false;

        return $this->registered[$handle]->extra[$key];
    }





    public function register_script( $handle, $src, $deps = array(), $ver = false, $in_footer = false ) {
    
        $registered = $this->add( $handle, $src, $deps, $ver );
        if ( $in_footer ) {
            $this->add_data( $handle, 'group', 1 );
        }
    
        return $registered;
    }



    public function enqueue_script( $handle, $src = '', $deps = array(), $ver = false, $in_footer = false ) {
    
        if ( $src || $in_footer ) {
            $_handle = explode( '?', $handle );
    
            if ( $src ) {
                $this->add( $_handle[0], $src, $deps, $ver );
            }
    
            if ( $in_footer ) {
                $this->add_data( $_handle[0], 'group', 1 );
            }
        }
    
        $this->enqueue( $handle );
	}
	



	public function print_scripts($handle = '')
	{
		$code = '';

		if(empty($handle))
		{
			foreach($this->queue as $handle)
			{
				$script = array_key_exists($handle, $this->registered) ? $this->registered[$handle] : array();

				if( !empty($script))
				{
					$extra = !empty($script->extra) ? $script->extra : array();
					$ver = !empty($script->ver) ? '?ver='.$script->ver : '';
					if(array_key_exists('data', $extra))
					{
						$code.= "\n<script type='text/javascript'>\n";*/
						//$code.= "/* <![CDATA[ */\n"; // not needed in HTML 5
						//$code.= $extra['data'];
						//$code.= "/* ]]> */\n";
						/*$code.= "</script>\n";
					}
					$code.= sprintf( "<script type='text/javascript'>\n%s\n</script>\n", $script->src.$ver );
				}
			}
		}
		
		return $code;
	}*/

}













class _IMC_Dependency {
	
	public $handle;
	public $src;
	public $deps = array();
	public $ver = false;
	public $args = null;  // Custom property, such as $in_footer or $media.
	public $extra = array();

	public function __construct() {
		@list( $this->handle, $this->src, $this->deps, $this->ver, $this->args ) = func_get_args();
		if ( ! is_array($this->deps) )
			$this->deps = array();
	}

	/**
	 * Add handle data.
	 *
	 * @param string $name The data key to add.
	 * @param mixed  $data The data value to add.
	 * @return bool False if not scalar, true otherwise.
	 */
	public function add_data( $name, $data ) {
		if ( !is_scalar($name) )
			return false;
		$this->extra[$name] = $data;
		return true;
	}

}
?>