<?php
require_once( IMGMCE_ABSPATH.'/include/classes/TF_Hook.php' );


function add_action($tag, $function_to_add, $priority = 10, $accepted_args = 1) 
{
	return add_filter($tag, $function_to_add, $priority, $accepted_args);
}


function add_filter( $tag, $function_to_add, $priority = 10, $accepted_args = 1 ) {
	global $tf_filter;
	if ( ! isset( $tf_filter[ $tag ] ) ) {
		$tf_filter[ $tag ] = new TF_Hook();
	}
	$tf_filter[ $tag ]->add_filter( $tag, $function_to_add, $priority, $accepted_args );
	return true;
}



function apply_filters( $tag, $value ) {
	global $tf_filter, $tf_current_filter;

	$args = array();

	// Do 'all' actions first.
	if ( isset($tf_filter['all']) ) {
		$tf_current_filter[] = $tag;
		$args = func_get_args();
		_tf_call_all_hook($args);
	}

	if ( !isset($tf_filter[$tag]) ) {
		if ( isset($tf_filter['all']) )
			array_pop($tf_current_filter);
		return $value;
	}

	if ( !isset($tf_filter['all']) )
		$tf_current_filter[] = $tag;

	if ( empty($args) )
		$args = func_get_args();

	// don't pass the tag name to tf_Hook
	array_shift( $args );

	$filtered = $tf_filter[ $tag ]->apply_filters( $value, $args );

	array_pop( $tf_current_filter );

	return $filtered;
}



function do_action($tag, $arg = '') {
	global $tf_filter, $tf_actions, $tf_current_filter;

	if ( ! isset($tf_actions[$tag]) )
		$tf_actions[$tag] = 1;
	else
		++$tf_actions[$tag];

	// Do 'all' actions first
	if ( isset($tf_filter['all']) ) {
		$tf_current_filter[] = $tag;
		$all_args = func_get_args();
		_tf_call_all_hook($all_args);
	}

	if ( !isset($tf_filter[$tag]) ) {
		if ( isset($tf_filter['all']) )
			array_pop($tf_current_filter);
		return;
	}

	if ( !isset($tf_filter['all']) )
		$tf_current_filter[] = $tag;

	$args = array();
	if ( is_array($arg) && 1 == count($arg) && isset($arg[0]) && is_object($arg[0]) ) // array(&$this)
		$args[] =& $arg[0];
	else
		$args[] = $arg;
	for ( $a = 2, $num = func_num_args(); $a < $num; $a++ )
		$args[] = func_get_arg($a);

	$tf_filter[ $tag ]->do_action( $args );

	array_pop($tf_current_filter);
}



function has_action($tag, $function_to_check = false) {
	return has_filter($tag, $function_to_check);
}

function has_filter($tag, $function_to_check = false) {
	global $tf_filter;

	if ( ! isset( $tf_filter[ $tag ] ) ) {
		return false;
	}

	return $tf_filter[ $tag ]->has_filter( $tag, $function_to_check );
}



function _tf_call_all_hook($args) {
	global $tf_filter;

	$tf_filter['all']->do_all_hook( $args );
}



function _tf_filter_build_unique_id($tag, $function, $priority) {
	global $tf_filter;
	static $filter_id_count = 0;

	if ( is_string($function) )
		return $function;

	if ( is_object($function) ) {
		// Closures are currently implemented as objects
		$function = array( $function, '' );
	} else {
		$function = (array) $function;
	}

	if (is_object($function[0]) ) {
		// Object Class Calling
		if ( function_exists('spl_object_hash') ) {
			return spl_object_hash($function[0]) . $function[1];
		} else {
			$obj_idx = get_class($function[0]).$function[1];
			if ( !isset($function[0]->tf_filter_id) ) {
				if ( false === $priority )
					return false;
				$obj_idx .= isset($tf_filter[$tag][$priority]) ? count((array)$tf_filter[$tag][$priority]) : $filter_id_count;
				$function[0]->tf_filter_id = $filter_id_count;
				++$filter_id_count;
			} else {
				$obj_idx .= $function[0]->tf_filter_id;
			}

			return $obj_idx;
		}
	} elseif ( is_string( $function[0] ) ) {
		// Static Calling
		return $function[0] . '::' . $function[1];
	}
}
?>